!
Subroutine rttov_iniscatt (&
      & errorstatus,       &! out
      & nlevels,           &! in
      & nchannels,         &! in
      & nprofiles,         &! in
      & chanprof,          &! in
      & frequencies,       &! in
      & profiles,          &! in  
      & cld_profiles,      &! in 
      & coef_rttov,        &! in
      & coef_scatt,        &! in
      & transmission,      &! in
      & calcemiss,         &! in
      & usenewcld,         &! in
      & usercfrac,         &! in
      & angles,            &! out
      & scatt_aux)          ! inout 

  !
  ! Description:
  ! Calculates some variables related to the input precipitation profile
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more painiscattrtners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  ! 
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !   1.0    09/2002   Initial version     (F. Chevallier)
  !   1.1    05/2003   RTTOV7.3 compatible (F. Chevallier)
  !   1.2    03/2004   Added polarimetry   (R. Saunders)
  !   1.3    08/2004   Polarimetry fixes   (U. O'Keeffe)
  !   1.4    11/2004   Clean-up            (P. Bauer)
  !   1.5    10/2005   Fixes for rttov8 indexing   (U. O'Keeffe)
  !   1.6    11/2005   Add errorstatus to arguments (J. Cameron)   
  !   1.7    11/2007   RTTOV9 / cleanup    (A. Geer)   
  !   1.8    03/2008   Revised cloud partitioning (A. Geer)  
  !   1.9    03/2009   Safety check on cloud fraction (A. Geer) 
  !   1.10   11/2009   User may supply the average cloud fraction (A. Geer)
  !   1.11   11/2009   RTTOV transmittances / optical depths come on levels (A Geer)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  ! 
  ! Declarations:
  ! Modules used:
  ! Imported Type Definitions:
   
  USE YOMHOOK, ONLY: LHOOK , DR_HOOK
  
  Use rttov_types, Only :     &
       & rttov_coef          ,&
       & rttov_scatt_coef    ,&
       & transmission_type   ,&
       & transmission_type_aux  ,&
       & geometry_Type       ,&
       & profile_scatt_aux   ,&
       & profile_Type        ,&
       & profile_cloud_Type  ,&
       & rttov_chanprof      ,&
       & rttov_options

  Use rttov_const, Only:      &
       & errorstatus_success, &
       & errorstatus_fatal,   &
       & gravity,             &
       & pressure_top,        &
       & rgp,                 &
       & rm,                  &
       & rho_rain,            &
       & rho_snow,            &
       & ccthres 

  Use parkind1, Only : jpim, jprb, jplm

  Implicit None
  
!* Subroutine arguments:
  Integer (Kind=jpim), Intent (in) :: nlevels   ! Number of levels
  Integer (Kind=jpim), Intent (in) :: nprofiles ! Number of profiles
  Integer (Kind=jpim), Intent (in) :: nchannels ! Number of channels*profiles=radiances
  Integer (Kind=jpim), Intent(out) :: errorstatus             ! Error return code
  Integer (Kind=jpim), Intent (in) :: frequencies (nchannels) ! Frequency indices
  Type(rttov_chanprof), Intent(in) :: chanprof    (nchannels) ! Channel and profile indices
  Logical (Kind=jplm), Intent (in) :: calcemiss   (nchannels) ! Emissivity flags
  Logical (Kind=jplm), Intent (in) :: usenewcld               ! New or old cloud partition
  Logical (Kind=jplm), Intent (in) :: usercfrac               ! User has supplied cloud fraction

  Type (profile_Type),        Intent (in)    :: profiles (nprofiles)     ! Atmospheric profiles
  Type (rttov_coef),          Intent (in)    :: coef_rttov               ! RTTOV Coefficients
  Type (rttov_scatt_coef),    Intent (in)    :: coef_scatt               ! RTTOV_SCATT Coefficients
  Type (profile_cloud_Type),  Intent (in)    :: cld_profiles (nprofiles) ! Cloud profiles 
  Type (transmission_Type),   Intent (in)    :: transmission             ! Transmittances and optical depths
  Type (geometry_Type),       Intent (out)   :: angles (nprofiles)       ! Zenith angles
  Type (profile_scatt_aux),   Intent (inout) :: scatt_aux                ! Auxiliary profile variables

!INTF_END

!* Local variables
  Integer (Kind=jpim) :: ilayer, iprof, ichan, iccmax
  Real    (Kind=jprb) :: p1, p2, pm, dp2dz, de2mr, zccmax

  Real (Kind=jprb), Dimension (nprofiles,nlevels)   :: presf  ! Pressure levels [hPa]
  Real (Kind=jprb), Dimension (nprofiles,nlevels+1) :: presfh ! Half-level pressure levels [hPa]
  Real (Kind=jprb), Dimension (nprofiles,nlevels)   :: dztop  ! Thickness of top half of level [km]
  Real (Kind=jprb), Dimension (nprofiles,nlevels)   :: dzbot  ! Thickness of bottom half of level [km]
  Real (Kind=jprb), Dimension (2:nlevels)           :: dzr    ! Thickness of RTTOV level [km]
  Real (Kind=jprb), Dimension (nchannels,nlevels)   :: od     ! Single layer optical depths on our levels
  Real (Kind=jprb), Dimension (nlevels+1)           :: od_rttov ! Single layer optical depths on RTTOV levels
  Real (Kind=jprb), Dimension (nchannels)           :: zod_up_cld   ! Optical depth from top of the atmosphere 

  Real    (Kind=jprb) :: hydro_weights(nlevels), hydro_column    ! Total hydrometeor amounts [g m^-2]

  Type (transmission_type_aux) :: transmissioncld  ! Clear+cloud transmittances with cloud
  Type(rttov_options)  :: opts

  Character (len=80) :: errMessage
  Character (len=15) :: NameOfRoutine = 'rttov_iniscatt '
  
  REAL(KIND=JPRB) :: ZHOOK_HANDLE
  
#include "rttov_mieproc.interface"
#include "rttov_iniedd.interface"
#include "rttov_calcemis_mw.interface"
#include "rttov_setgeometry.interface"
#include "rttov_errorreport.interface"

  !- End of header --------------------------------------------------------

  if (lhook) call dr_hook('RTTOV_INISCATT',0_jpim,zhook_handle)

  errorstatus = errorstatus_success

  ! NB this only truly needed if using FASTEM 3 
  allocate (transmissioncld % tau_surf (0:0,nchannels))

  de2mr =  1.0E+05_JPRB * rm / rgp
  dp2dz = -1.0E-03_JPRB * rgp / gravity / rm 

  scatt_aux % ext (:,:) = 0.0_JPRB
  scatt_aux % ssa (:,:) = 0.0_JPRB
  scatt_aux % asm (:,:) = 0.0_JPRB

!* Security on user-defined pressures
  Do iprof = 1, nprofiles
     Do ilayer = 1, nlevels
        If (profiles (iprof) % p (ilayer) >= pressure_top) Then
            presf (iprof,ilayer) = profiles (iprof) % p  (ilayer)
        else
            presf (iprof,ilayer) = pressure_top
        endif
    Enddo
    Do ilayer = 1, nlevels + 1
       If (cld_profiles (iprof) % ph (ilayer) >= pressure_top) Then
           presfh (iprof,ilayer) = cld_profiles (iprof) % ph (ilayer)
       else
           presfh (iprof,ilayer) = pressure_top
       endif
    Enddo
  Enddo

!* Geometric variables
  Call rttov_setgeometry ( &
    & opts,       & ! in
    & profiles,        & ! in
    & coef=coef_rttov, & ! in
    & angles=angles)     ! out

!* Temperature at layer boundaries (K)
  Do iprof = 1, nprofiles
     scatt_aux % tbd (iprof,nlevels+1) = profiles (iprof) % s2m % t
     scatt_aux % tbd (iprof,1)         = profiles (iprof) % t (1)
  Enddo

  Do ilayer = 1, nlevels-1
     Do iprof = 1, nprofiles
        p1 = presf  (iprof,ilayer+1)
        p2 = presf  (iprof,ilayer  )
        pm = presfh (iprof,ilayer+1)

        scatt_aux % tbd (iprof,ilayer+1) =  profiles (iprof) % t (ilayer+1) &
                                       & + (profiles (iprof) % t (ilayer)   & 
                                       & -  profiles (iprof) % t (ilayer+1)) &
                                       & / log(p2/p1) * log(pm/p1) 
     Enddo
  Enddo

  !* Initialise cloud and rain properties of the cloudy/rainy column
  scatt_aux % clw   (:,:) = 0.0_JPRB
  scatt_aux % ciw   (:,:) = 0.0_JPRB
  scatt_aux % totalice   (:,:) = 0.0_JPRB
  scatt_aux % rain  (:,:) = 0.0_JPRB
  scatt_aux % sp    (:,:) = 0.0_JPRB

  if(usenewcld) then

    ! Weighted average partitioning between "cloudy" and "clear" columns
    Do ilayer=1,nlevels
      Do iprof = 1, nprofiles  
        scatt_aux % clw  (iprof,ilayer) = cld_profiles (iprof) % clw  (ilayer) 
        if ( cld_profiles (iprof) % use_totalice ) then
          scatt_aux % totalice (iprof,ilayer) = cld_profiles (iprof) % totalice (ilayer) 
        else
          scatt_aux % ciw  (iprof,ilayer) = cld_profiles (iprof) % ciw  (ilayer) 
          scatt_aux % sp   (iprof,ilayer) = cld_profiles (iprof) % sp   (ilayer) 
        endif 
        scatt_aux % rain (iprof,ilayer) = cld_profiles (iprof) % rain (ilayer) 
      Enddo
    Enddo

  else

    ! Maximum cloud fraction partitioning between "cloudy" and "clear" columns
    Do iprof = 1, nprofiles  
      zccmax = 0.0_JPRB
      iccmax = 0

      Do ilayer = 1, nlevels
        if (cld_profiles (iprof) % cc (ilayer) > zccmax) then
          zccmax = cld_profiles (iprof) % cc (ilayer)
          iccmax = ilayer
        end if 
      end do 
      scatt_aux % cfrac (iprof) = zccmax
    Enddo

    do ilayer=1,nlevels
      do iprof = 1, nprofiles  
        If (scatt_aux % cfrac (iprof) > ccthres) Then
          scatt_aux % clw  (iprof,ilayer) = cld_profiles (iprof) % clw  (ilayer) / scatt_aux % cfrac (iprof)
          scatt_aux % rain (iprof,ilayer) = cld_profiles (iprof) % rain (ilayer) / scatt_aux % cfrac (iprof)
          if ( cld_profiles (iprof) % use_totalice ) then
            scatt_aux % totalice (iprof,ilayer) = cld_profiles (iprof) % totalice (ilayer) / scatt_aux % cfrac (iprof)
          else
            scatt_aux % ciw  (iprof,ilayer) = cld_profiles (iprof) % ciw  (ilayer) / scatt_aux % cfrac (iprof)
            scatt_aux % sp   (iprof,ilayer) = cld_profiles (iprof) % sp   (ilayer) / scatt_aux % cfrac (iprof)
          endif
        Endif
      enddo
    enddo

  endif

!* Nadir heights (km)
  Do ilayer = nlevels, 1, -1
     Do iprof = 1, nprofiles
        p1 = presfh (iprof,ilayer+1)
        p2 = presfh (iprof,ilayer  )
        pm = presf  (iprof,ilayer  )

        If (p1 <= p2) then
           errorstatus = errorstatus_fatal
           Write( errMessage, '( "iniscatt : problem with user-defined pressure layering")' )
           Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
           if (lhook) call dr_hook('RTTOV_INISCATT',1_jpim,zhook_handle)
           Return
        End If

        scatt_aux % dz (iprof,ilayer) = dp2dz * Log(p2/p1) * profiles (iprof) % t (ilayer)

        dzbot (iprof,ilayer) = dp2dz * Log(pm/p1) * profiles (iprof) % t (ilayer)
        dztop (iprof,ilayer) = dp2dz * Log(p2/pm) * profiles (iprof) % t (ilayer)

     Enddo
  Enddo

!* Get single-layer optical depths (at nadir and in hPa-1) and put onto model half levels
  Do ichan = 1, nchannels
     iprof = chanprof(ichan) % prof

! Top RTTOV level to space    
     od_rttov (1)         = -1.0_jprb * log( transmission % tau_levels (1,ichan) )     
     Do ilayer = 2, nlevels 
        od_rttov (ilayer) = log( transmission % tau_levels (ilayer-1,ichan) ) &
                        & - log( transmission % tau_levels (ilayer,ichan) )
     Enddo
! Surface to bottom RTTOV (full pressure) level
     od_rttov (nlevels+1) = log( transmission % tau_levels (nlevels,ichan) ) &
                        & - log( transmission % tau_total (ichan) )

! RTTOV layer thickness
     Do ilayer = 2, nlevels  
       dzr(ilayer) = dzbot(iprof,ilayer-1) + dztop(iprof,ilayer)
     Enddo

! Re-allocate optical depths between half pressure levels        
     od (ichan,1)         = od_rttov(1) &
                        & + od_rttov(2) * dzbot(iprof,1) / dzr(2)     
     Do ilayer = 2, nlevels - 1  
        od (ichan,ilayer) = od_rttov(ilayer)   * dztop(iprof,ilayer) / dzr(ilayer) &
                        & + od_rttov(ilayer+1) * dzbot(iprof,ilayer) / dzr(ilayer+1) 
     Enddo
     od (ichan,nlevels)   = od_rttov(nlevels)  * dztop(iprof,nlevels) / dzr(nlevels) &
                        & + od_rttov(nlevels+1) 
      
  Enddo

!* Change units
  Do ilayer = 1, nlevels

!* Optical depths in km-1 and at nadir
     Do ichan = 1, nchannels
        iprof = chanprof(ichan) % prof
     
        scatt_aux % ext (ichan,ilayer) = od (ichan,ilayer) &
          & / scatt_aux % dz (iprof,ilayer) * angles (iprof) % coszen 

        if (scatt_aux % ext (ichan,ilayer) < 1.0E-10_JPRB) scatt_aux % ext (ichan,ilayer) = 1.0E-10_JPRB
     Enddo

!* Condensate from g/g to g/m^3
     Do iprof = 1, nprofiles
        scatt_aux % clw (iprof,ilayer) = scatt_aux % clw (iprof,ilayer) * presf (iprof,ilayer) &
                                        & * de2mr / profiles (iprof) % t (ilayer) 
        scatt_aux % ciw (iprof,ilayer) = scatt_aux % ciw (iprof,ilayer) * presf (iprof,ilayer) &
                                        & * de2mr / profiles (iprof) % t (ilayer) 
        scatt_aux % totalice (iprof,ilayer) = scatt_aux % totalice (iprof,ilayer) * presf (iprof,ilayer) &
                                        & * de2mr / profiles (iprof) % t (ilayer) 
    
!* Rates from kg/m^2/s to g/m^3
        scatt_aux % rain (iprof,ilayer) = scatt_aux % rain (iprof,ilayer) / rho_rain
        scatt_aux % sp   (iprof,ilayer) = scatt_aux % sp   (iprof,ilayer) / rho_snow

        scatt_aux % rain (iprof,ilayer) = scatt_aux % rain (iprof,ilayer) * 3600.0_JPRB 
        scatt_aux % sp   (iprof,ilayer) = scatt_aux % sp   (iprof,ilayer) * 3600.0_JPRB

        if (scatt_aux % rain (iprof,ilayer) > 0.0_JPRB) &
          & scatt_aux % rain (iprof,ilayer) = (scatt_aux % rain (iprof,ilayer) * &
          & coef_scatt % conv_rain (1))**(coef_scatt % conv_rain (2)) 
        if (scatt_aux % sp   (iprof,ilayer) > 0.0_JPRB) &
          & scatt_aux % sp   (iprof,ilayer) = (scatt_aux % sp   (iprof,ilayer) * &
          & coef_scatt % conv_sp   (1))**(coef_scatt % conv_sp   (2))
     enddo
  Enddo

  if(usenewcld) then 

    scatt_aux % cfrac (:)   = 0.0_JPRB

    Do iprof = 1, nprofiles

      if( usercfrac ) then

        !* User-supplied cloud fraction
        scatt_aux % cfrac (iprof) = cld_profiles (iprof) % cfrac

      else

        !* Calculate a hydrometeor-weighted average cloudy sky fraction 

        !* Partial column of hydrometeors in g/m^2
        hydro_weights(:) = &
          & ( scatt_aux % rain (iprof,:) + scatt_aux % sp (iprof,:) &
          &   + scatt_aux % ciw (iprof,:)  + scatt_aux % clw (iprof,:)  &
          &   + scatt_aux % totalice (iprof,:) ) &
          & * scatt_aux % dz (iprof,:)
        hydro_column = sum( hydro_weights(:) )

        !* Weighted mean cloud fraction
        if (hydro_column > 1e-10_JPRB) then
          scatt_aux % cfrac (iprof) = &
            & sum(hydro_weights(:) * cld_profiles (iprof) % cc (:)) / hydro_column
          if ( scatt_aux % cfrac (iprof) < 0.0_JPRB ) scatt_aux % cfrac (iprof) = 0.0_JPRB
          if ( scatt_aux % cfrac (iprof) > 1.0_JPRB ) scatt_aux % cfrac (iprof) = 1.0_JPRB
        else
          scatt_aux % cfrac (iprof) = 0.0_JPRB
        endif

      endif

      !* Partition all cloud and rain into the cloudy column
      If (scatt_aux % cfrac (iprof) > ccthres) Then
         scatt_aux % clw  (iprof,:) = scatt_aux % clw  (iprof,:) / scatt_aux % cfrac (iprof)
         scatt_aux % ciw  (iprof,:) = scatt_aux % ciw  (iprof,:) / scatt_aux % cfrac (iprof)
         scatt_aux % totalice  (iprof,:) = scatt_aux % totalice  (iprof,:) / scatt_aux % cfrac (iprof)
         scatt_aux % rain (iprof,:) = scatt_aux % rain (iprof,:) / scatt_aux % cfrac (iprof)
         scatt_aux % sp   (iprof,:) = scatt_aux % sp   (iprof,:) / scatt_aux % cfrac (iprof)
      else
         scatt_aux % clw  (iprof,:) = 0.0_JPRB
         scatt_aux % ciw  (iprof,:) = 0.0_JPRB
         scatt_aux % totalice  (iprof,:) = 0.0_JPRB
         scatt_aux % rain (iprof,:) = 0.0_JPRB
         scatt_aux % sp   (iprof,:) = 0.0_JPRB
      Endif
    Enddo

  endif

!* Cloud/rain absorption/scattering parameters
  Call rttov_mieproc (      &
       & nlevels,           &! in
       & nchannels,         &! in
       & nprofiles,         &! in
       & frequencies,       &! in
       & chanprof%prof,     &! in
       & profiles,          &! in
       & coef_scatt,        &! in
       & scatt_aux)          ! inout 

!* Scattering parameters for Eddington RT
  Call rttov_iniedd (       &
       & nlevels,           &! in
       & nchannels ,        &! in
       & nprofiles ,        &! in
       & chanprof%prof,     &! in
       & angles    ,        &! in
       & scatt_aux)          ! inout 

!* Surface emissivities
  zod_up_cld (:) = 0.0_JPRB
  
  Do ichan = 1, nchannels
     iprof = chanprof(ichan) % prof
     
     Do ilayer = 1, nlevels
        zod_up_cld (ichan) = zod_up_cld (ichan) + scatt_aux % ext (ichan,ilayer) * scatt_aux % dz (iprof,ilayer)
     Enddo
     
     if (zod_up_cld (ichan) >= 30.0_JPRB) zod_up_cld (ichan) = 30.0_JPRB
     transmissioncld % tau_surf (0,ichan) = Exp(-1.0_JPRB * zod_up_cld (ichan) / angles (iprof) % coszen)
  Enddo
  
  Call rttov_calcemis_mw (     &
       & opts,                 &! in
       & profiles,             &! in
       & angles,               &! in
       & coef_rttov,           &! in
       & chanprof,             &! in
       & transmissioncld,      &! in
       & calcemiss,            &! in
       & scatt_aux % ems_cld,  &! inout
       & scatt_aux % ref_cld,  &! out
       & errorstatus          ) ! inout 
  
!* Hemispheric emissivity (= Fastem's effective emissivity)
  scatt_aux % ems_bnd (:) = scatt_aux % ems_cld (:)
  scatt_aux % ref_bnd (:) = scatt_aux % ref_cld (:)

!* Deallocate
  deallocate (transmissioncld % tau_surf)

  if (lhook) call dr_hook('RTTOV_INISCATT',1_jpim,zhook_handle)

End Subroutine rttov_iniscatt
