module mod_mie

! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2002, EUMETSAT, All Rights Reserved.

! Module for use with create_tables_spectra and associated subroutines

! Current Code Owner: SAF NWP

! History:
! Version   Date        Comment
! -------   ----        -------
!           04/09/2008   transferred to fcm (Amy Doherty)
!           12/03/2010   restructuring (Alan Geer)

use parkind1,  only: jprb, jpim

implicit none


!* constants
real (kind=jprb)  , parameter :: pi          = 3.141592654_jprb
real (kind=jprb)  , parameter :: cm2km       = 1.0e+05_jprb
real (kind=jprb)  , parameter :: gm32gcm3    = 1.0e-06_jprb

! Density (g.cm^-3)
real (kind=jprb)  , parameter :: density_wat = 1.0_jprb
real (kind=jprb)  , parameter :: density_air = 1.225e-03_jprb
real (kind=jprb)  , parameter :: density_ice = 0.917_jprb 
real (kind=jprb)  , parameter :: density_lmt = 0.05_jprb 

complex (kind=jprb), parameter :: perm_air    = (1.0006_jprb, 0.0_jprb)

!* constants for melting routines
integer (kind=jpim), parameter :: max_iter = 1000
real    (kind=jprb), parameter :: eps      = 0.001_jprb
      
real (kind=jprb)  , parameter :: onethird = 0.3333_jprb

real (kind=jprb)  , parameter :: t_ml = 275.0_jprb       ! T at bottom of melting layer in K
real (kind=jprb)  , parameter :: t_fl = 273.15_jprb      ! temperature at FL in k
real (kind=jprb)  , parameter :: rh   = 0.98_jprb        ! relative humidity

real (kind=jprb)  , parameter :: ka = 5.69e-05_jprb      ! heat conductivity in cal/cm/s/c
real (kind=jprb)  , parameter :: dv = 0.211_jprb         ! diffusivity of vapor in air in cm*cm/s
real (kind=jprb)  , parameter :: mw = 18.0_jprb          ! molecular weight of water in g/mol
real (kind=jprb)  , parameter :: le = 797.3_jprb         ! latent heat of evaporation in cal/g
real (kind=jprb)  , parameter :: lm = 79.7_jprb          ! latent heat of melting in cal/g
real (kind=jprb)  , parameter :: r  = 8.314e+07_jprb     ! gas constant in erg/mol/k
real (kind=jprb)  , parameter :: va = 0.1346_jprb        ! kinematic viscosity of air in cm*cm/s

real (kind=jprb)  , parameter :: g       = 981.0_jprb    ! gravity [cm/s^2]
real (kind=jprb)  , parameter :: cd_snow = 1.2_jprb      ! drag coeff. snow []
real (kind=jprb)  , parameter :: cd_grpl = 1.0_jprb      ! drag coeff. graupel []
real (kind=jprb)  , parameter :: cd_hail = 0.6_jprb      ! drag coeff. hail []

!* parameterizations
real (kind=jprb), parameter :: lwc2rr_a = 20.95_jprb
real (kind=jprb), parameter :: lwc2rr_b = 1.12_jprb

real    (kind=jprb), parameter :: wc_slope  = 0.01_jprb
integer (kind=jpim), parameter :: wc_offset = 301

integer (kind=jpim), parameter :: lmax    = 500 !Maximum length of filename/path and strings in text files
integer (kind=jpim), parameter :: nconfig = 12  !Number of lines in the configuration section of the Mie table file

!* Mie-tables
!*************
!*
integer (kind=jpim), parameter :: n_temp = 70
integer (kind=jpim), parameter :: n_lwc  = 401
integer (kind=jpim), parameter :: n_dia  = 100

!* Hydrometeor types
!********************
!*
integer (kind=jpim), parameter :: n_type = 7
integer (kind=jpim), parameter :: i_rain      = 1
integer (kind=jpim), parameter :: i_snow      = 2
integer (kind=jpim), parameter :: i_graupel   = 3
integer (kind=jpim), parameter :: i_aggregate = 4
integer (kind=jpim), parameter :: i_clw       = 5
integer (kind=jpim), parameter :: i_ciw       = 6
integer (kind=jpim), parameter :: i_totalice  = 7

integer (kind=jpim), parameter :: ctype_max = 11
character (len=ctype_max) :: ctype(n_type) = (/&
& 'rain       ',&
& 'snow       ',&
& 'graupel    ',&
& 'aggregate  ',&
& 'cloud-water',&
& 'cloud-ice  ',&
& 'total_ice  '/)

!* Density, min/max diameters (cm)
real (kind=jprb), dimension (n_type) :: density_liq = (/ 1.00,   0.10,   0.40,   0.90,   1.0,    0.9,    0.5    /)
real (kind=jprb), dimension (n_type) :: d_min       = (/ 0.01,   0.01,   0.05,   0.50,   0.0005, 0.0005, 0.0005 /)
real (kind=jprb), dimension (n_type) :: d_max       = (/ 1.00,   2.00,   0.30,   2.00,   0.0100, 0.0100, 2.00   /)  
real (kind=jprb), dimension (n_type) :: mue         = (/ 0.0,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0    /)
real (kind=jprb), dimension (n_type) :: n0_fac      = (/ 1.0,    0.5,    0.5,    0.5,    0.0,    0.0,    0.5    /)
real (kind=jprb), dimension (n_type) :: temp_offset = (/ 233.0,  203.0,  203.0,  203.0,  233.0,  203.0,  203.0  /)
real (kind=jprb), dimension (n_type) :: conv_a      = (/ 20.89,  29.51,    0.0,    0.0,    0.0,    0.0,    0.0  /)
real (kind=jprb), dimension (n_type) :: conv_b      = (/  1.15,   1.10,    0.0,    0.0,    0.0,    0.0,    0.0  /) ! AJGDB why diff to above?

contains

function get_lwc(iwc)
  real (kind=jprb) :: get_lwc
  integer (kind=jpim), intent(in) :: iwc
  get_lwc = 10.0_jprb ** (wc_slope * (iwc - wc_offset))
end function get_lwc

end module mod_mie
