program rttov_scatt_make_coef

! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2002, EUMETSAT, All Rights Reserved.

! Description:
! Main program for generating tables of scattering coefficients for use with RTTOV.
! Mie theory is used (spherical particles are assumed), with the delta approximation to account
! for the two stream solution of the rttovscatt radiative transfer model.
! Coefficients (extinction coefficient, single scattering albedo and asymmetry parameter)
! are generated for each of 14 instruments
! 1:SMOS, 2:AMSR, 3:SSM/I, 4:TMI, 5:SSMIS, 6:AMSU-A, 7:AMSU-B, 8:MHS, 9:WINDSAT, 10:EGPM, 11:NAST-M, 12:PR, 13:DPR, 14:CR
! at 70 temperatures, 401 water contents, and 100 particle diameters. 
! These tables are then read into rttovscatt
! (available with RTTOV version 9 onwards) in the subrouting rttov_readscattcoeffs.F90.
! 
! for more details on the instruments see channels.dat_all
!
! The code is set up to generate the coefficients for different types of hydrometeor:
!
! EITHER snow and ice OR total ice are then used in rttovscatt using the "use_totalice" flag
! in rttovscatt_test.F90.
! 

! Method:
! 

! Current Code Owner: SAF NWP

! History:
! Version   Date        Comment
! -------   ----        -------
!           04/09/2008   transferred to fcm (Amy Doherty)
!   1.2     23/04/2009   changed field PSD treatement to Field '07 instead of Field '05
!                        giving choice between tropical and mid latitude regime (Amy Doherty)
!   1.3     12/03/2010   Optimisation and restructuring (Alan Geer)

!**************************************************************************

use parkind1, only: jprb, jpim, jplm

use mod_mie, only: n_temp, n_type, n_lwc, n_dia, &
                   pi, density_air, density_ice, cm2km, &
                   i_totalice, temp_offset, &
                   i_snow, i_graupel, i_aggregate, get_lwc, lmax

implicit none

real (kind=jprb)    :: f_ghz, fc_ghz, df_ghz
real (kind=jprb)    :: temp, lwc
real (kind=jprb)    :: perm_re, perm_im
real (kind=jprb)    :: wavelength, znorm
real (kind=jprb)    :: ext, sct, asm, bsct
integer :: nargs
integer (kind=jpim) :: i_freq, j_freq, i_type, i_mtype, i_temp, i_lwc, i_dia, n_band, i_band
integer (kind=jpim) :: n_freq, n_sensor
integer (kind=jpim) :: dens, psd
complex (kind=jprb) :: k, perm_wat
logical (kind=jplm) :: ll_melt, ll_dsd
logical :: dir_exists
character :: regime
character(len=lmax) :: line
character(len=lmax) :: output_dir
character(len=lmax) :: arg
character(len=lmax) :: input_file

integer (kind=jpim) :: m_type 
integer (kind=jpim), allocatable, dimension (:) :: id_type 

real (kind=jprb), dimension (:,:,:,:), allocatable :: ext_tab, ssa_tab, asm_tab, zef_tab

!* Interface blocks
#include "convert_mietable.interface"
#include "mie_one_wc.interface"
#include "perm_water.interface"

!-------------------------------------------------------------------------------

!------------------------
! Read configuration file
!------------------------

! Use default file name 'channels.dat' unless one is specified on the command
! line.

nargs = command_argument_count()
if ( nargs > 0 ) then
  call get_command_argument(1,input_file)
else
  input_file = 'channels.dat'
end if

open(10, file = input_file, status = 'old')

print '(a)', 'Reading configuration file ' // trim(input_file) // '...'

! Look for the header

do
  read (10,*) line
  if ( line == 'HEADER' ) exit
enddo

read (10,*)
read (10,'(a500)') output_dir
read (10,*) n_freq
read (10,*) n_sensor
read (10,*) m_type
allocate(id_type(m_type))
read (10,*) id_type 
read (10,*)
read (10,*)
read (10,*) ll_melt 
read (10,*)
read (10,*)
read (10,*)
read (10,*)
read (10,*) ll_dsd
read (10,*)
read (10,*) dens
read (10,*) psd
read (10,*) regime

print '(a)', 'Config:'
print '(a)', ' Output dir = ' // trim(output_dir)
print '(a,i0)', ' Number of frequencies = ', n_freq
print '(a,i0)', ' Number of sensors = ', n_sensor
print '(a,i0)', ' Number of hydrometeor types = ', m_type
print '(a,l1)', ' Melting layer = ', ll_melt
print '(a,l1)', ' Parameterized n0 = ', ll_melt
print '(a,i0)', ' Density = ', dens
print '(a,i0)', ' PSD = ', psd
print '(a)', ' Regime = ' // regime

! Find frequencies

do
  read (10,*) line
  if ( line == 'FREQUENCIES' ) exit
end do

read (10,*)

!---------------------
! Calculate Mie tables
!---------------------

print '(a)', 'Calculating Mie tables...'

! Initialize Mie table arrays

allocate(ext_tab(n_freq,n_type,n_temp,n_lwc)) 
allocate(ssa_tab(n_freq,n_type,n_temp,n_lwc)) 
allocate(asm_tab(n_freq,n_type,n_temp,n_lwc))
allocate(zef_tab(n_freq,n_type,n_temp,n_lwc))
ssa_tab = 0.0
ext_tab = 0.0
asm_tab = 0.0
zef_tab = 0.0

! Loop (1) over frequencies

do i_freq = 1, n_freq

  ! read in from channels.dat
  read (10,*) j_freq, fc_ghz, n_band, df_ghz

  print '(a,i0,a,i0,a,f8.3,a)', &
    'Freq ', i_freq, ' of ', n_freq, ': ', fc_ghz, ' GHz'

  !* radar reflectivity norm. factor
  wavelength = 29.997_jprb / fc_ghz  
  call perm_water (fc_ghz, 273.0_jprb, perm_re, perm_im)
  perm_wat = cmplx (perm_re, perm_im, jprb)
  k        = (perm_wat - 1.0_jprb) / (perm_wat + 2.0_jprb)
  znorm    = 1.0e+12 * wavelength ** 4.0_jprb / (pi ** 5.0_jprb * abs (k) ** 2.0_jprb)

  ! Loop (2) over bands

  do i_band = 1, n_band

    ! If the instrument channel has side bands, calculate freqs of these based
    ! on info read in from channels.dat then calculate the scattering coefs for
    ! each band add them together and divide by number of bands to get coefs
    ! for channel.

    if (n_band > 1) then
      f_ghz = fc_ghz + (2 * i_band - 3) * df_ghz
    else
      f_ghz = fc_ghz
    end if

    ! Loop (3) over hydrometeor types

    do i_mtype = 1, m_type

      i_type = id_type(i_mtype)

      write(*,'(4(a,i0))') ' Band ', i_band, ' of ', n_band, &
        '; hydrometeor type ', i_type, ' of ', n_type

      ! Loop (4) over temperatures

      do i_temp = 1, n_temp

        temp = temp_offset(i_type) + i_temp

        ! Loop over water content. Allow this loop to use multiple threads to
        ! speed computation.

        !$OMP PARALLEL DO PRIVATE( lwc, ext, sct, asm, bsct )
        do i_lwc = 1, n_lwc

          lwc = get_lwc(i_lwc)

          call mie_one_wc( lwc, temp, wavelength, f_ghz, i_type, psd, dens, regime, &
            & (ll_melt .and. i_temp == 70), ll_dsd, ext, sct, asm, bsct)

          ssa_tab(i_freq,i_type,i_temp,i_lwc) = ssa_tab(i_freq,i_type,i_temp,i_lwc) + sct / ext 
          ext_tab(i_freq,i_type,i_temp,i_lwc) = ext_tab(i_freq,i_type,i_temp,i_lwc) + ext * cm2km
          asm_tab(i_freq,i_type,i_temp,i_lwc) = asm_tab(i_freq,i_type,i_temp,i_lwc) + asm
          zef_tab(i_freq,i_type,i_temp,i_lwc) = zef_tab(i_freq,i_type,i_temp,i_lwc) + bsct
 
        end do 
        !$OMP END PARALLEL DO

      end do !(4)

    end do !(3)

  end do !(2)

  ssa_tab(i_freq,:,:,:) =  ssa_tab(i_freq,:,:,:) / n_band
  ext_tab(i_freq,:,:,:) =  ext_tab(i_freq,:,:,:) / n_band
  asm_tab(i_freq,:,:,:) =  asm_tab(i_freq,:,:,:) / n_band
  where( zef_tab(i_freq,:,:,:) > 0.0 )
    zef_tab(i_freq,:,:,:) =  10.0_jprb * log10 (znorm * zef_tab(i_freq,:,:,:) / n_band)
  end where

end do !(1)

close(10)

call convert_mietable(input_file, ext_tab, ssa_tab, asm_tab, dens, psd, regime)

deallocate(asm_tab)
deallocate(ssa_tab)
deallocate(ext_tab)
deallocate(zef_tab)

end program rttov_scatt_make_coef
