!
SUBROUTINE rttov_print_info (coefs, lu, text)
  ! Description:
  ! print information about RTTOV and (optionally)
  !   about the supplied coefficient file structure
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2007, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0       05/04/2011  New code 
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:

  ! Imported Type Definitions:
  USE rttov_types, ONLY :       &
        & rttov_coefs

  USE parkind1, ONLY : jpim

!INTF_OFF
  USE rttov_global, ONLY : &
      error_unit

  USE rttov_const, ONLY :       &
        & version,              &
        & release,              &
        & minor_version,        &
        & platform_name,        &
        & inst_name,            &
        & sensor_name,          &
        & sensor_id_ir,         &
        & sensor_id_hi,         &
        & gas_name,             &
        & gas_unit_name
      
  USE parkind1, ONLY : jprb, jplm
!INTF_ON

  IMPLICIT NONE

  ! subroutine arguments
  TYPE( rttov_coefs ), INTENT(IN), OPTIONAL :: coefs  ! coefs structure
  INTEGER(KIND=jpim),  INTENT(IN), OPTIONAL :: lu     ! logical unit for print
  CHARACTER(LEN=*),    INTENT(IN), OPTIONAL :: text   ! additional text to print

!INTF_END
  INTEGER(KIND=jpim)  :: iu        ! logical unit for print
  CHARACTER(LEN=128)  :: tmp_text  ! temporary string for formatting output
  INTEGER(KIND=jpim)  :: int_kind  ! Standard integer
  REAL(KIND=jprb)     :: real_kind ! Standard real
  INTEGER(KIND=jpim)  :: g, gas_id ! Indexes

  iu = error_unit
  IF( PRESENT( lu ) ) iu = lu

  IF( PRESENT( text ) ) THEN
    WRITE(iu,'(/,a,a)') "RTTOV information: ",TRIM(text)
  ELSE
    WRITE(iu,'(/,a)') "RTTOV information"
  END IF
  
  WRITE(tmp_text,'(i2,".",i1,".",i1)') version, release, minor_version
  WRITE(iu,'(2x,a,a)')    "RTTOV library version     ", TRIM(tmp_text)
  WRITE(tmp_text,'(es19.9)') huge(real_kind)
  WRITE(iu,'(2x,a,a)')    "Maximum real size         ", TRIM(ADJUSTL(tmp_text))
  WRITE(tmp_text,'(i25)') huge(int_kind)
  WRITE(iu,'(2x,a,a)')    "Maximum integer size      ", TRIM(ADJUSTL(tmp_text))
  
  IF( PRESENT(coefs) ) THEN
    WRITE(iu,'(/,a)') "Coefficient file"
    WRITE(tmp_text,'(i4.4,"/",i2.2,"/",i2.2)') coefs % coef % id_creation_date(1), &
                                             & coefs % coef % id_creation_date(2), &
                                             & coefs % coef % id_creation_date(3)
    WRITE(iu,'(2x,a,a)')  "Creation date             ", TRIM(tmp_text)
    WRITE(tmp_text,'(i2)') coefs % coef % id_comp_lvl
    WRITE(iu,'(2x,a,a)')  "Coefficient file version  ", TRIM(ADJUSTL(tmp_text))
    WRITE(iu,'(a)',advance='yes')
    
    WRITE(tmp_text,'(a," - ",i2)') TRIM(platform_name(coefs % coef % id_platform)), coefs % coef % id_sat
    WRITE(iu,'(2x,a,a)')  "Platform                  ", TRIM(tmp_text)
    WRITE(iu,'(2x,a,a)')  "Instrument                ", TRIM(inst_name(coefs % coef % id_inst))
    WRITE(iu,'(2x,a,a)')  "Sensor type               ", TRIM(sensor_name(coefs % coef % id_sensor))
    WRITE(tmp_text,'(f9.2)') coefs % coef % fc_sat_height
    WRITE(iu,'(2x,a,a)')  "Nominal satellite height  ", TRIM(ADJUSTL(tmp_text))
    WRITE(tmp_text,'(i4)') coefs % coef % fmv_chn
    WRITE(iu,'(2x,a,a)')  "Number of channels        ", TRIM(ADJUSTL(tmp_text))
    WRITE(iu,'(2x,a,l1)') "Zeeman coefficient file?  ", coefs % coef % inczeeman
    IF (coefs % coef % id_comp_pc == 0) THEN
      WRITE(iu,'(2x,a,l1)') "PC compatibility?         ", .FALSE._jplm
    ELSE
      WRITE(iu,'(2x,a,l1)') "PC compatibility?         ", .TRUE._jplm
    END IF
    WRITE(iu,'(a)',advance='yes')
    
    WRITE(tmp_text,'(i2)') coefs % coef % fmv_model_ver
    WRITE(iu,'(2x,a,a)')  "Fast model version        ", TRIM(ADJUSTL(tmp_text))
    WRITE(tmp_text,'(i3)') coefs % coef % nlevels
    WRITE(iu,'(2x,a,a)')  "Number of levels          ", TRIM(ADJUSTL(tmp_text))
    WRITE(tmp_text,'(i3)') coefs % coef % fmv_gas
    WRITE(iu,'(2x,a,a)')  "Number of gases           ", TRIM(ADJUSTL(tmp_text))
    WRITE(iu,'(2x,4a12)')  "Gas name    ", "# predictors","  # coeffs  ","units       "
    DO g = 1, coefs % coef % fmv_gas
      gas_id = coefs % coef % fmv_gas_id(g)  
      WRITE(iu,'(2x,a12,i8,4x,i8,4x,a12)') gas_name(gas_id),                          &
                                         & coefs % coef % fmv_var(g),                 &
                                         & coefs % coef % fmv_coe(g),                 &
                                         & gas_unit_name(coefs % coef % gaz_units(g))
    END DO
    WRITE(iu,'(a)',advance='yes')
    
    IF ( coefs % coef % id_sensor == sensor_id_ir .OR. &
         coefs % coef % id_sensor == sensor_id_hi ) THEN
      WRITE(tmp_text,'(2x,i3)') coefs % coef % ssirem_ver
      WRITE(iu,'(2x,a,a)')  "SSIREM model version      ",TRIM(ADJUSTL(tmp_text))
    ELSE
      WRITE(tmp_text,'(2x,i3)') coefs % coef % fastem_ver
      WRITE(iu,'(2x,a,a)')  "FASTEM model version      ",TRIM(ADJUSTL(tmp_text))
      WRITE(tmp_text,'(a,i3.3,a)') "(2x,",coefs % coef % fmv_chn,"i3)"
      WRITE(iu,'(2x,a)')    "Channel polarisations"
      WRITE(iu,tmp_text) coefs % coef % fastem_polar(:)
    END IF
    
  END IF
  
END SUBROUTINE rttov_print_info
