Program example_pc_fwd
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2010, EUMETSAT, All Rights Reserved.
  !
  !     *************************************************************
  !
  !     TEST PROGRAM FOR PRINCIPAL COMPONENT RTTOV SUITE FORWARD MODEL ONLY
  !          RTTOV VERSION 10
  ! To run this program you must have the following files
  ! either resident in the same directory or set up as a
  ! symbolic link:
  !   rtcoef_platform_id_sensor.dat --  coefficient file to match
  !   pccoef_platform_id_sensor.dat --  coefficient file to match
  !   the sensor you request in the header of the program
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date        Comment
  ! -------   ----        -------
  !  1.0    18/10/2010   P. Brunel
  !
  ! Code Description:
  !   Language:          Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
#include "throw.h"
  Use rttov_const, Only :     &
       & surftype_sea,        &
       & watertype_ocean_water

  Use rttov_types, Only :   &
       & rttov_options,     &
       & rttov_coefs,       &
       & profile_Type,      &
       & transmission_Type, &
       & radiance_Type,     &
       & rttov_chanprof,    &
       & rttov_pccomp

  Use parkind1, Only : jpim, jprb, jplm

  Use rttov_unix_env, Only: rttov_exit

  Use rttov_chain, only : print_array
  !
  Implicit None

  Character*(*), Parameter :: Gformat = 'G15.6E3'
  !
#include "rttov_direct.interface"
#include "rttov_read_coefs.interface"
#include "rttov_init_coefs.interface"
#include "rttov_dealloc_coefs.interface"
#include "rttov_alloc_rad.interface"
#include "rttov_alloc_transmission.interface"
#include "rttov_alloc_prof.interface"
#include "rttov_alloc_pccomp.interface"
#include "rttov_errorreport.interface"
#include "rttov_print_profile.interface"
#include "rttov_get_pc_predictindex.interface"
  !--------------------------
  !
  Type(rttov_options)               :: opts               ! Options structure
  Type(rttov_coefs)                 :: coefs              ! Coefficients structure
  Type(rttov_chanprof), Allocatable :: chanprof(:)
  Type(profile_Type)  , Allocatable :: profiles(:)
  Logical(Kind=jplm)  , Allocatable :: calcemis(:)
  Real(Kind=jprb),      Allocatable :: emissivity (:)
  Real(Kind=jprb),      Allocatable :: emissivity_out (:)
  Integer(Kind=jpim),   Allocatable :: channels_rec (:)
  Type(transmission_Type)           :: transmission
  Type(radiance_Type)               :: radiance
  TYPE(rttov_pccomp)                :: pccomp
  Integer(Kind=jpim),   Pointer     :: predictindex(:)
  
  Integer(Kind=jpim)  :: ERR

  Integer(Kind=jpim) :: asw
  Integer(Kind=jpim) :: i,j,k
  Integer(Kind=jpim) :: file_id = 40

  Character(Len=4)   :: inst_name
  Character(Len=128) :: file_coef
  Character(Len=128) :: file_pccoef
  Integer(Kind=jpim) :: nchannels
  Integer(Kind=jpim) :: nchannels_per_profile
  Integer(Kind=jpim) :: nchannels_rec_per_profile
  Integer(Kind=jpim) :: ipcreg
  Integer(Kind=jpim) :: npcscores_per_profile
  Integer(Kind=jpim) :: nprofiles
  Integer(Kind=jpim) :: nlevels
  Integer(Kind=jpim) :: prof

  !- End of header --------------------------------------------------------
TRY

  ! Please note that for Principal Component calculations the following
  ! are mandatory:
  ! Profiles over sea only
  ! Surface emissivities are always calculated
  ! Clear atmosphere only 
  ! Channels for PC predictors are fixed according to the number of predictors
  ! Number of predictors are function of the regression set (ipcreg)
  ! RTTOV optical depth regression coefficients should be compatible
  ! Input profile shall contain default values for trace gases, they are overwritten
  !    for the calculation of optical depth predictors
  ! Refraction should always be set

  ! This test includes the calculation of the reconstructed radiances for the
  ! full instrument channels. This is an option in the PC calculation.
  ! use "opts % addradrec = .false." to remove this option


  Write(0,*) "Enter instrument: 'iasi' or 'airs'"
  Read(*,*) inst_name

  If (inst_name == "iasi") Then
    ! IASI Metop-2
    file_coef   = "rtcoef_metop_2_iasi.dat"
    file_pccoef = "pccoef_metop_2_iasi.dat"
    nchannels_rec_per_profile = 8461_jpim   ! number of reconstructed radiances for one profile
    npcscores_per_profile     = 400_jpim    ! number of principal compoments for one profile
    ipcreg                    = 4_jpim      ! regression set among [1..4]
  Else If (inst_name == "airs") Then
    ! AIRS 
    file_coef   = "rtcoef_eos_2_airs.dat"
    file_pccoef = "pccoef_eos_2_airs.dat"
    nchannels_rec_per_profile = 2378_jpim   ! number of reconstructed radiances for one profile
    npcscores_per_profile     = 400_jpim    ! number of principal compoments for one profile
    ipcreg                    = 3_jpim      ! regression set among [1..3]
  Else
    ERR = 1
  End If
  THROWM( ERR .NE. 0 , "Unknown input instrument: must be 'iasi' or 'airs'")
  Write(0,*) "Running example_pc_fwd for "//inst_name
  
  ! Number of profiles in one run (maximum 3 in this test)
  nprofiles = 3_jpim

  ! Option set for reading coefficients
  opts % addpc      = .true.
  opts % addclouds  = .false.
  opts % addaerosl  = .false.

  ! Read coefficients, all channels. This is the recommanded way
  Call rttov_read_coefs(                   &
        & err, coefs, opts,                &
        & file_coef   = Trim(file_coef),   &
        & file_pccoef = Trim(file_pccoef)  )
  THROWM( ERR .NE. 0 , "rttov_read_coefs fatal error")

  Call rttov_init_coefs ( err, opts, coefs )
  THROWM( ERR .NE. 0 , "rttov_init_coefs fatal error")

  ! plus some important options for PC calc
  ! Regression set
  opts % ipcreg     = ipcreg
  ! If user wants reconstructed radiances
  opts % addradrec  = .true.

  !----------------------------------------------
  ! Example on use of rttov_get_pc_predictindex:
  !
  Nullify(predictindex)
!   Call rttov_get_pc_predictindex(ERR, opts, predictindex, file_pccoef=file_pccoef)
!   THROWM( ERR .NE. 0 , "rttov_get_pc_predictindex fatal error")
!   nchannels_per_profile = SIZE(predictindex)
  !----------------------------------------------
  
  !
  ! Channels for predictors are driven by the regression set
  nchannels_per_profile = SIZE(coefs%coef_pccomp%pcreg(opts%ipcreg)%predictindex)
  nchannels             = nchannels_per_profile     * nprofiles

  !Pack channels and emmissivity arrays
  Allocate(                   &
  & chanprof(nchannels),      &
  & emissivity(nchannels),    &
  & emissivity_out(nchannels),&
  & calcemis(nchannels),      &
  & stat= ERR )
  THROWM( ERR .NE. 0 , "allocate chanprof array")

  ! Initialise chanprof structure (regression channels)
  DO prof = 1, nprofiles
    i = 1 + (prof-1)*nchannels_per_profile
    j = prof * nchannels_per_profile
    
    !----------------------------------------------
    ! Example on use of rttov_get_pc_predictindex:
    !
!     chanprof(i:j)%chan = predictindex(:)
    !----------------------------------------------
    
    chanprof(i:j)%chan = coefs%coef_pccomp%pcreg(opts%ipcreg)%predictindex(:)
    chanprof(i:j)%prof = prof
  ENDDO

  ! For AddRec initialise channels_rec array
  ! here we calculate all instrument spectrum
  IF ( opts % addradrec ) THEN
    Allocate(                      &
    & channels_rec(nchannels_rec_per_profile), &
    & stat= ERR )
    THROWM( ERR .NE. 0 , "allocate channels_rec array")

    channels_rec = (/ (k, k=1,nchannels_rec_per_profile) /)
  ENDIF

  ! Initialise options structure
  ! Mandatory values:
  opts % addrefrac  = .True.
  opts % addsolar   = .False.
  opts % addclouds  = .False.
  opts % addaerosl  = .False.
  opts % ozone_data = .True.
  opts % co2_data   = .True.
  opts % n2o_data   = .True.
  opts % ch4_data   = .True.
  opts % co_data    = .True.
  opts % clw_data   = .False.

  ! options
  opts % addinterp                   = .True.
  opts % do_checkinput               = .True.
  opts % verbose_checkinput_warnings = .True.

  !==========================================================
  ! Here manage your own profiles
  ! All profiles should have the same number of levels
  !==========================================================
  ! Input profile is on 44 levels; this is an example
  ! interpolation flag is set
  nlevels = 44_jpim

  Allocate(                &
    & profiles(nprofiles), &
    & stat= ERR )
  THROWM( ERR .NE. 0 , "allocate profiles array")

  asw = 1 ! allocate
  profiles(:) % nlevels = nlevels
  call rttov_alloc_prof( &
      & err,             &
      & nprofiles,       &
      & profiles,        &
      & nlevels,         &
      & opts,            &
      & asw,             &
      & init = .true._jplm )
  THROWM( ERR .NE. 0 , "allocate profile")

  DO prof = 1, nprofiles
    ! fill atmospheric values for profile
    Call defaultprofile( prof, profiles(prof), ERR)
    THROWM( ERR .NE. 0 , "default profile erroro=")

    ! 2 meter air variables
    profiles(prof) % s2m % p  = profiles(prof) % p(nlevels)
    profiles(prof) % s2m % p  = 1007.3

    profiles(prof) % s2m % t  = profiles(prof) % t(nlevels)
    profiles(prof) % s2m % q  = profiles(prof) % q(nlevels)
    profiles(prof) % s2m % o  = profiles(prof) % o3(nlevels)
    profiles(prof) % s2m % u  = 5.0_jprb
    profiles(prof) % s2m % v  = 2.0_jprb
    profiles(prof) % s2m % wfetc  = 100.0_jprb

    ! Skin variables
    profiles(prof) % skin % t  = profiles(prof) % t(nlevels)

    ! Cloud variables... Needs a valid could top pressure
    profiles(prof) % ctp        = 500.0_jprb
    profiles(prof) % cfraction  =   0.0_jprb

    ! Other variables for geometry
    profiles(prof) % zenangle    = 45.0_jprb
    profiles(prof) % azangle     = 45.0_jprb
    profiles(prof) % latitude    = 45.0_jprb
    profiles(prof) % elevation   =  0.0_jprb
    profiles(prof) % sunzenangle =  0.0_jprb
    profiles(prof) % sunazangle  =  0.0_jprb

    ! surface type SEA only
    profiles(prof) % skin % surftype  = surftype_sea
    profiles(prof) % skin % watertype = watertype_ocean_water

  ENDDO
  !==========================================================
  ! End of manage your own profile
  !==========================================================

  ! allocate radiance results arrays with number of channels
  asw = 1 ! allocate
  call rttov_alloc_rad( &
      & err,            &
      & nchannels,      &
      & radiance,       &
      & nlevels-1_jpim,      &
      & asw)
  THROWM( ERR .NE. 0 , "allocate radiances")

  ! allocate transmittance structure
  call rttov_alloc_transmission( &
      & err,                     &
      & transmission,            &
      & nlevels-1_jpim,               &
      & nchannels,               &
      & asw,                     &
      & init = .true._jplm)
  THROWM( ERR .NE. 0 , "allocate transmittances")

  ! Take care of the allocation on number of pcscores
  ! multiplied by the number of profiles
  IF( opts%addradrec )then
    ! Take care of the allocation on number of reconstructed channels
    ! multiplied by the number of profiles
    call rttov_alloc_pccomp (err, pccomp, npcscores_per_profile * nprofiles, 1_jpim, init = .true._jplm, &
                                & nchannels_rec = nchannels_rec_per_profile * nprofiles)
  else
    call rttov_alloc_pccomp (err, pccomp,npcscores_per_profile * nprofiles, 1_jpim, init = .true._jplm)
  endif
  THROWM(err.ne.0, "allocate PC-Comp")

  ! For PC calcemis should allway be true
  emissivity(:)    = 0._jprb
  calcemis(:)      = .true.

  ! Call RTTOV forward model with the 2 PC specific arguments 
  ! pccomp and channels_rec
  call rttov_direct(         &
        & err,               &! out   error flag
        & chanprof,          &! in    channel and profile index structure
        & opts,              &! in    options structure
        & profiles,          &! in    profile array
        & coefs,             &! in    coefficients strucutre
        & calcemis,          &! in    flag for intermal emissivity calc
        & emissivity,        &! in    input emissivities per channel
        & emissivity_out,    &! out   emissivities used by RTTOV per channel
        & transmission,      &! inout computed transmittances
        & radiance,          &! inout computed radiances 
        & pccomp       = pccomp,            &! PC scores
        & channels_rec = channels_rec       )! list of channels for reconstructed radiances
  THROWM( ERR .NE. 0 , "rttov_direct error")

  ! Print results
  Open(file_id, file="output_example_pc_fwd_"//inst_name//".dat", form='formatted')
  
  ! PC Scores:
  DO prof = 1, nprofiles

    Write( file_id, * )' Profile ', prof,'  ',TRIM(profiles(prof)%id)
    call rttov_print_profile (profiles(prof) , lu=file_id)

    Write( file_id, * )' PC Scores '
    i = 1 + (prof-1)*npcscores_per_profile
    j = prof * npcscores_per_profile

    Write( file_id, '(A," = (")' ) 'PCCOMP%PCSCORES'
    Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', &
            &  A1 = pccomp%pcscores(i:j) )
    Write( file_id, * ) ')'

    IF ( opts % addradrec ) THEN
      i = 1 + (prof-1)*nchannels_rec_per_profile
      j = prof * nchannels_rec_per_profile
      Write( file_id, * )
      Write( file_id, * ) 'Clear radiances reconstructed using principal components'
      Write( file_id, '(A," = (")' ) 'PCCOMP%CLEAR_PCCOMP'
      Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', &
              &  A1 = pccomp % clear_pccomp (i:j) )
      Write( file_id, * ) ')'

      Write( file_id, * )'Brightness temp equivalent to radiances reconstructed using PCs'
      Write( file_id, '(A," = (")' ) 'PCCOMP%BT_PCCOMP'
      Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', &
              &  A1 = pccomp % bt_pccomp (i:j) )
      Write( file_id, * ) ')'
      Write( file_id, * )
    ENDIF

  ENDDO
  
  Close(file_id)

  IF ( opts % addradrec ) THEN
    deallocate(         &
      & channels_rec,   &
      & stat=ERR)
    THROWM( ERR .NE. 0 , "Dealloc channels_rec")
  ENDIF

  ! Deallocate arrays
  deallocate(         &
    & chanprof,       &
    & emissivity,     &
    & emissivity_out, &
    & calcemis,       &
    & stat=ERR)
  THROWM( ERR .NE. 0 , "Dealloc emmis")

  asw = 0 ! deallocate radiance arrays
  call rttov_alloc_rad (err,nchannels,radiance,nlevels-1_jpim,asw)
  THROWM( ERR .NE. 0 , "dealloc radiances error")


  asw = 0 ! deallocate transmission arrays
  call rttov_alloc_transmission (err,transmission,nlevels-1_jpim,nchannels,asw)
  THROWM( ERR .NE. 0 , "dealloc transmittances error")
  
  asw = 0 ! deallocate profile arrays
  call rttov_alloc_prof (err,nprofiles,profiles,nlevels,opts,asw)
  THROWM( ERR .NE. 0 , "dealloc profile structure error")

  deallocate(         &
    & profiles,       &
    & stat=ERR)
  THROWM( ERR .NE. 0 , "dealloc profiles array error")

  asw = 0 ! deallocate PC arrays
  call rttov_alloc_pccomp (err, pccomp,npcscores_per_profile * nprofiles, asw)
  THROWM( ERR .NE. 0 , "dealloc pccomp error")

  Call rttov_dealloc_coefs (err, coefs)
  THROWM( ERR .NE. 0 , "dealloc coefficients error")

  If (Associated(predictindex)) Deallocate(predictindex)
PCATCH

CONTAINS

Subroutine defaultprofile( ip, profile, ERR)
USE parkind1, ONLY : jpim, jprb
USE rttov_types, ONLY :  &
       & profile_Type

Integer(Kind=jpim), intent(in)     :: ip
Type(profile_Type), Intent(inout)  :: profile
Integer(Kind=jpim), Intent(out)    :: err

Real(Kind=jprb), Parameter :: prof1(4,44) = Reshape( (/ &
    &  0.005   , 219.10   ,    4.819 ,  1.       , &
    &  0.1     , 219.10   ,    4.819 ,  1.       , &
    &  0.29    , 249.82   ,    4.819 ,  1.2573   , &
    &  0.69    , 255.99   ,    4.819 ,  1.7804   , &
    &  1.42    , 257.54   ,    4.819 ,  2.8865   , &
    &  2.611   , 251.72   ,    4.819 ,  3.2868   , &
    &  4.407   , 243.48   ,    4.819 ,  3.2444   , &
    &  6.95    , 235.02   ,    4.819 ,  3.1582   , &
    &  10.37   , 229.33   ,    4.819 ,  2.985    , &
    &  14.81   , 226.62   ,    4.819 ,  2.8164   , &
    &  20.4    , 223.05   ,    4.819 ,  2.053    , &
    &  27.26   , 218.62   ,    4.819 ,  1.4848   , &
    &  35.51   , 215.08   ,    4.819 ,  1.3399   , &
    &  45.29   , 212.13   ,    4.819 ,  1.2732   , &
    &  56.73   , 207.95   ,    4.819 ,  1.0438   , &
    &  69.97   , 202.51   ,    4.819 ,  0.8993   , &
    &  85.18   , 194.34   ,    4.819 ,  0.7404   , &
    &  102.05  , 193.15   ,    4.819 ,  0.5553   , &
    &  122.04  , 198.05   ,    4.819 ,  0.3604   , &
    &  143.84  , 205.49   ,    4.819 ,  0.2674   , &
    &  167.95  , 212.48   ,    4.819 ,  0.1596   , &
    &  194.36  , 219.84   ,    4.819 ,  0.0818   , &
    &  222.94  , 226.74   ,    4.819 ,  0.049    , &
    &  253.71  , 233.39   ,   27.455 ,  0.0455   , &
    &  286.6   , 240.02   ,  392.513 ,  0.0451   , &
    &  321.5   , 246.66   ,  603.394 ,  0.0455   , &
    &  358.28  , 253.05   ,  615.583 ,  0.0454   , &
    &  396.81  , 258.76   ,  514.178 ,  0.0451   , &
    &  436.95  , 263.40   , 1383.774 ,  0.0414   , &
    &  478.54  , 267.21   , 3650.321 ,  0.0377   , &
    &  521.46  , 271.02   , 4687.105 ,  0.0358   , &
    &  565.54  , 275.13   , 4283.691 ,  0.035    , &
    &  610.6   , 278.31   , 8288.053 ,  0.0341   , &
    &  656.43  , 281.06   ,11187.702 ,  0.0332   , &
    &  702.73  , 283.33   ,12868.921 ,  0.0324   , &
    &  749.12  , 285.48   ,15718.135 ,  0.0308   , &
    &  795.09  , 287.82   ,17900.259 ,  0.0291   , &
    &  839.95  , 290.44   ,19007.149 ,  0.0274   , &
    &  882.8   , 292.76   ,20385.323 ,  0.026    , &
    &  922.46  , 294.82   ,22529.266 ,  0.025    , &
    &  957.44  , 296.76   ,26704.048 ,  0.0264   , &
    &  985.88  , 299.19   ,28355.334 ,  0.0293   , &
    &  1005.43 , 300.97   ,29218.883 ,  0.0316   , &
    &  1013.25 , 301.64   ,29512.537 ,  0.0324   /), &
    &  (/4,44/) )

Real(Kind=jprb), Parameter :: prof2(4,44) = Reshape( (/ &
    &  0.005   , 220.8207  ,    4.8187 , 1.0093  , &
    &  0.1     , 217.8650  ,    4.7844 , 0.9959  , &
    &  0.29    , 250.6653  ,    4.7998 , 1.2511  , &
    &  0.69    , 253.7369  ,    4.7970 , 1.7916  , &
    &  1.42    , 255.3679  ,    4.8229 , 2.8679  , &
    &  2.611   , 249.4346  ,    4.8651 , 3.2828  , &
    &  4.407   , 245.0812  ,    4.7904 , 3.2135  , &
    &  6.95    , 236.4042  ,    4.8147 , 3.1749  , &
    &  10.37   , 227.0830  ,    4.8614 , 3.0009  , &
    &  14.81   , 226.7524  ,    4.8282 , 2.8424  , &
    &  20.4    , 223.7137  ,    4.7848 , 2.0674  , &
    &  27.26   , 220.7502  ,    4.8592 , 1.4983  , &
    &  35.51   , 215.8609  ,    4.8521 , 1.3477  , &
    &  45.29   , 212.1343  ,    4.8549 , 1.2777  , &
    &  56.73   , 208.1886  ,    4.8027 , 1.0430  , &
    &  69.97   , 204.0757  ,    4.8368 , 0.8970  , &
    &  85.18   , 195.4570  ,    4.8494 , 0.7408  , &
    &  102.05  , 194.0215  ,    4.7747 , 0.5545  , &
    &  122.04  , 196.6286  ,    4.8254 , 0.3614  , &
    &  143.84  , 204.1835  ,    4.8034 , 0.2672  , &
    &  167.95  , 212.8559  ,    4.8553 , 0.1598  , &
    &  194.36  , 217.7217  ,    4.8440 , 0.0811  , &
    &  222.94  , 226.1675  ,    4.8494 , 0.0494  , &
    &  253.71  , 234.6350  ,   27.3347 , 0.0453  , &
    &  286.6   , 239.4540  ,  390.7022 , 0.0450  , &
    &  321.5   , 247.9289  ,  608.1976 , 0.0454  , &
    &  358.28  , 253.6229  ,  615.5206 , 0.0453  , &
    &  396.81  , 256.5237  ,  516.1252 , 0.0455  , &
    &  436.95  , 265.1056  , 1373.7437 , 0.0417  , &
    &  478.54  , 265.0388  , 3662.4136 , 0.0380  , &
    &  521.46  , 269.0656  , 4667.8531 , 0.0357  , &
    &  565.54  , 275.1754  , 4307.7994 , 0.0351  , &
    &  610.6   , 275.6223  , 8339.1127 , 0.0338  , &
    &  656.43  , 278.3564  ,11188.3153 , 0.0331  , &
    &  702.73  , 285.8965  ,12876.6294 , 0.0323  , &
    &  749.12  , 287.1966  ,15831.5111 , 0.0306  , &
    &  795.09  , 288.7593  ,18070.2821 , 0.0293  , &
    &  839.95  , 287.7945  ,18986.6286 , 0.0277  , &
    &  882.8   , 289.8721  ,20277.0206 , 0.0260  , &
    &  922.46  , 296.9486  ,22716.8341 , 0.0248  , &
    &  957.44  , 298.1029  ,26892.5627 , 0.0263  , &
    &  985.88  , 298.3362  ,28556.2628 , 0.0294  , &
    &  1005.43 , 300.4058  ,29094.1562 , 0.0315  , &
    &  1013.25 , 299.9594  ,29387.3986 , 0.0322  /), &
    &  (/4,44/) )


Real(Kind=jprb), Parameter :: prof3(4,44) = Reshape( (/ &
    &  0.005   , 220.7674  ,    4.7942 , 0.9929  , &
    &  0.1     , 220.2342  ,    4.7761 , 1.0078  , &
    &  0.29    , 251.1791  ,    4.8050 , 1.2472  , &
    &  0.69    , 254.9204  ,    4.7854 , 1.7912  , &
    &  1.42    , 256.1672  ,    4.8055 , 2.8710  , &
    &  2.611   , 249.7300  ,    4.7944 , 3.2590  , &
    &  4.407   , 245.7110  ,    4.8146 , 3.2318  , &
    &  6.95    , 235.8188  ,    4.8243 , 3.1665  , &
    &  10.37   , 230.4257  ,    4.8607 , 2.9887  , &
    &  14.81   , 227.2357  ,    4.8127 , 2.8156  , &
    &  20.4    , 223.5094  ,    4.8074 , 2.0669  , &
    &  27.26   , 217.2120  ,    4.8074 , 1.4848  , &
    &  35.51   , 214.2215  ,    4.8007 , 1.3278  , &
    &  45.29   , 210.5076  ,    4.8404 , 1.2631  , &
    &  56.73   , 207.3676  ,    4.8290 , 1.0432  , &
    &  69.97   , 204.1003  ,    4.8152 , 0.9064  , &
    &  85.18   , 194.6565  ,    4.7823 , 0.7390  , &
    &  102.05  , 191.2873  ,    4.7786 , 0.5586  , &
    &  122.04  , 198.1376  ,    4.8188 , 0.3607  , &
    &  143.84  , 204.3428  ,    4.8290 , 0.2671  , &
    &  167.95  , 211.0740  ,    4.8291 , 0.1597  , &
    &  194.36  , 220.9275  ,    4.8512 , 0.0811  , &
    &  222.94  , 226.0081  ,    4.7774 , 0.0494  , &
    &  253.71  , 235.4179  ,   27.6944 , 0.0454  , &
    &  286.6   , 239.2147  ,  392.3201 , 0.0455  , &
    &  321.5   , 248.6537  ,  607.2051 , 0.0453  , &
    &  358.28  , 250.6469  ,  612.2394 , 0.0457  , &
    &  396.81  , 256.1948  ,  513.7568 , 0.0448  , &
    &  436.95  , 265.0983  , 1379.5267 , 0.0416  , &
    &  478.54  , 269.0250  , 3658.9335 , 0.0378  , &
    &  521.46  , 270.2334  , 4654.1998 , 0.0357  , &
    &  565.54  , 274.7947  , 4316.6087 , 0.0347  , &
    &  610.6   , 277.4974  , 8364.4976 , 0.0338  , &
    &  656.43  , 282.9268  ,11155.8636 , 0.0335  , &
    &  702.73  , 281.9223  ,12987.1005 , 0.0323  , &
    &  749.12  , 285.1242  ,15783.1554 , 0.0307  , &
    &  795.09  , 286.0604  ,17969.6335 , 0.0290  , &
    &  839.95  , 292.8764  ,18957.9967 , 0.0275  , &
    &  882.8   , 294.5459  ,20325.4726 , 0.0258  , &
    &  922.46  , 297.3540  ,22424.7803 , 0.0252  , &
    &  957.44  , 295.6403  ,26510.6526 , 0.0264  , &
    &  985.88  , 300.6364  ,28366.5125 , 0.0294  , &
    &  1005.43 , 303.9092  ,29377.2089 , 0.0316  , &
    &  1013.25 , 300.3420  ,29234.3876 , 0.0322  /), &
    &  (/4,44/) )

TRY

    IF( ip == 1 ) THEN
      profile % id = 'Test profile standard44lev_allgas_seaonly/001'
      profile%p   = prof1(1,:)
      profile%t   = prof1(2,:)
      profile%q   = prof1(3,:)
      profile%o3  = prof1(4,:)
    ELSEIF ( ip == 2 ) THEN
      profile % id = 'Test profile standard44lev_allgas_seaonly/002'
      profile%p   = prof2(1,:)
      profile%t   = prof2(2,:)
      profile%q   = prof2(3,:)
      profile%o3  = prof2(4,:)
    ELSEIF ( ip == 3 ) THEN
      profile % id = 'Test profile standard44lev_allgas_seaonly/003'
      profile%p   = prof3(1,:)
      profile%t   = prof3(2,:)
      profile%q   = prof3(3,:)
      profile%o3  = prof3(4,:)
    ELSE
      ERR = 1_jpim 
      THROWM( ERR .NE. 0 , "invalid profile number")
    ENDIF

    ! PC calculations :
    ! Any value for CO2 N2O CH4 CO because all are overwritten
    ! by reference profile in setpredictors9
    profile%co2 = 1.0e-4_jprb
    profile%n2o = 1.0e-4_jprb
    profile%ch4 = 1.0e-4_jprb
    profile%co  = 1.0e-4_jprb

CATCH
End Subroutine

End Program example_pc_fwd
