!
SUBROUTINE rttov_k_ad( &
            & errorstatus,      &
            & chanprof,         &
            & opts,             &
            & profiles,         &
            & profiles_k,       &
            & coefs,            &
            & calcemis,         &
            & emissivity,       &
            & emissivity_k,     &
            & emissivity_out,   &
            & transmission,     &
            & radiancedata,     &
            & pccomp,           &
            & profiles_k_pc,    &
            & profiles_k_rec,   &
            & channels_rec)
! Description:
!   Compute jacobian using adjoint code
!
! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! Method:
!
! Current Code Owner: SAF NWP
!
! History:
! Version   Date     Comment
! -------   ----     -------
! 2010/03 Code cleaning Pascal Brunel, Philippe Marguinaud
!
!
! Code Description:
!   Language:           Fortran 90.
!   Software Standards: "European Standards for Writing and
!     Documenting Exchangeable Fortran 90 Code".
!INTF_OFF
#include "throw.h"
!INTF_ON

! Imported Type Definitions:
  USE rttov_types, ONLY :  &
       & rttov_coefs,       &
       & rttov_options,     &
       & profile_Type,      &
       & transmission_Type, &
       & rttov_chanprof,    &
       & radiance_Type,     & 
       & rttov_pccomp
  USE parkind1, ONLY : jpim, jprb, jplm

  IMPLICIT NONE
!subroutine arguments:
  TYPE(profile_Type  )   , INTENT(IN)   , TARGET :: profiles  (:)
  TYPE(rttov_chanprof)   , INTENT(IN)            :: chanprof  (:)
  TYPE(profile_Type  )   , INTENT(INOUT), TARGET :: profiles_k(size(chanprof))
! for calls from RTTOV_CLD_K and RTTOV_SCATT_K.
  TYPE(rttov_options )   , INTENT(IN)            :: opts
  TYPE(rttov_coefs   )   , INTENT(IN)            :: coefs
  LOGICAL(KIND=jplm)     , INTENT(IN)            :: calcemis        (size(chanprof))
  INTEGER(KIND=jpim)     , INTENT(OUT)           :: errorstatus
  REAL   (KIND=jprb)     , INTENT(IN)            :: emissivity      (size(chanprof))
  REAL   (KIND=jprb)     , INTENT(INOUT)         :: emissivity_k    (size(chanprof))
  REAL   (KIND=jprb)     , INTENT(OUT)           :: emissivity_out  (size(chanprof))
  TYPE(transmission_Type), INTENT(INOUT)         :: transmission                    ! in because of meme allocation
  TYPE(radiance_Type    ), INTENT(INOUT)         :: radiancedata                    ! in because of meme allocation
  TYPE(rttov_pccomp     ), OPTIONAL     , INTENT(INOUT)         :: pccomp
  TYPE(profile_Type     ), OPTIONAL     , INTENT(INOUT), TARGET :: profiles_k_pc(:)
  TYPE(profile_Type     ), OPTIONAL     , INTENT(INOUT), TARGET :: profiles_k_rec(:)
  INTEGER(KIND=jpim)     , OPTIONAL     , INTENT(IN)            :: channels_rec(:)
!INTF_END

#include "rttov_ad.interface"
#include "rttov_alloc_prof.interface"
#include "rttov_alloc_rad.interface"
#include "rttov_alloc_pccomp.interface"
#include "rttov_init_prof.interface"
#include "rttov_copy_prof.interface"
#include "rttov_alloc_transmission.interface"
#include "rttov_init_pccomp.interface"
#include "rttov_init_rad.interface"
#include "rttov_init_transmission.interface"
#include "rttov_errorreport.interface"

  TYPE(rttov_pccomp)      :: pccomp_ad
  TYPE(profile_Type), TARGET :: profiles_ad(size(profiles))
  REAL(KIND=jprb)         :: emissivity_ad    (size(chanprof))
  REAL(KIND=jprb)         :: emissivity_out_ad(size(chanprof))
  TYPE(transmission_Type) :: transmission_ad
  TYPE(radiance_Type    ) :: radiancedata_ad
  INTEGER(KIND=jpim)      :: ichan          , err
  INTEGER(KIND=jpim) :: nprofiles
  INTEGER(KIND=jpim) :: nchannels
  INTEGER(KIND=jpim) :: npcscores
  INTEGER(KIND=jpim) :: nchannels_rec
  INTEGER(KIND=jpim) :: ipcscore
  INTEGER(KIND=jpim) :: ichannels_rec
  INTEGER(KIND=jpim) :: iprof

TRY


  nprofiles = size(profiles)
  nchannels = size(chanprof)

  npcscores = -1_jpim
  nchannels_rec = -1_jpim

  IF (opts%addpc) THEN
    npcscores = size(pccomp%pcscores)
    IF (opts%addradrec) THEN
      nchannels_rec = size(pccomp%bt_pccomp)
    ENDIF
  ENDIF

  IF (opts%addpc) THEN
    CALL rttov_alloc_pccomp (err, pccomp_ad, npcscores, 1_jpim, nchannels_rec = nchannels_rec)
    THROW(err.ne.0)
  ENDIF

  CALL rttov_alloc_prof( &
        & err,                                                           &
        & nprofiles,                                                     &
        & profiles_ad,                                                   &
        & profiles(1)%nlevels,                                           &
        & opts,                                                          &
        & 1_jpim,                                                        &
        & coefs = coefs,                                                 &
        & init = .TRUE._jplm)
  THROW(err.ne.0)

  CALL rttov_alloc_rad( &
        & err,                          &
        & nchannels,                    &
        & radiancedata_ad,              &
        & profiles(1)%nlevels - 1_jpim, &
        & 1_jpim)
  THROW(err.ne.0)

  CALL rttov_alloc_transmission(err, transmission_ad, profiles(1)%nlevels-1_jpim, nchannels, 1_jpim)
  THROW(err.ne.0)

!
! Regular channels
!
  DO ichan = 1, nchannels

    emissivity_ad                = 0._jprb
    emissivity_out_ad            = 0._jprb

    CALL rttov_init_transmission (transmission_ad)
    CALL rttov_init_rad (radiancedata_ad)
    IF (opts%addpc) THEN
      radiancedata_ad%clear(ichan) = 1._jprb
    ELSE IF (opts%switchrad) THEN
      radiancedata_ad%bt(ichan) = 1._jprb
    ELSE
      radiancedata_ad%total(ichan) = 1._jprb
    ENDIF
      
    CALL rttov_init_prof (profiles_ad)

    IF (opts%addpc) THEN
      CALL rttov_init_pccomp (pccomp_ad)
    ENDIF

    CALL rttov_ad(                     &
          & errorstatus,               &
          & chanprof,                  &
          & opts,                      &
          & profiles,                  &
          & profiles_ad,               &
          & coefs,                     &
          & calcemis,                  &
          & emissivity,                &
          & emissivity_ad,             &
          & emissivity_out,            &
          & emissivity_out_ad,         &
          & transmission,              &
          & transmission_ad,           &
          & radiancedata,              &
          & radiancedata_ad,           &
          & pccomp       = pccomp,     &
          & pccomp_ad    = pccomp_ad,  &
          & channels_rec = channels_rec)

    IF (errorstatus /= 0) THEN
      err = errorstatus_fatal
      THROW(err.ne.0)
    ENDIF

    iprof = chanprof(ichan)%prof
    CALL rttov_copy_prof (profiles_k(ichan:ichan), profiles_ad(iprof:iprof))
    emissivity_k(ichan) = emissivity_ad(ichan)

  ENDDO

  IF (opts%addpc) THEN

    IF (opts%addradrec) THEN

!
! Reconstructed radiance
!

      DO ichannels_rec = 1, nchannels_rec
      
        emissivity_ad                = 0._jprb
        emissivity_out_ad            = 0._jprb

        CALL rttov_init_transmission (transmission_ad)
        CALL rttov_init_rad (radiancedata_ad)
      
        CALL rttov_init_pccomp (pccomp_ad)

        IF (opts%switchrad) THEN
          pccomp_ad%bt_pccomp(ichannels_rec) = 1._jprb
        ELSE
          pccomp_ad%clear_pccomp(ichannels_rec) = 1._jprb
        ENDIF
      
        CALL rttov_init_prof (profiles_ad)
 
        CALL rttov_ad(                     &
              & errorstatus,               &
              & chanprof,                  &
              & opts,                      &
              & profiles,                  &
              & profiles_ad,               &
              & coefs,                     &
              & calcemis,                  &
              & emissivity,                &
              & emissivity_ad,             &
              & emissivity_out,            &
              & emissivity_out_ad,         &
              & transmission,              &
              & transmission_ad,           &
              & radiancedata,              &
              & radiancedata_ad,           &
              & pccomp       = pccomp,     &
              & pccomp_ad    = pccomp_ad,  &
              & channels_rec = channels_rec)
      
        IF (errorstatus /= 0) THEN
          err = errorstatus_fatal
          THROW(err.ne.0)
        ENDIF
      
        iprof = 1 + ((nprofiles * (ichannels_rec-1))/nchannels_rec)
        CALL rttov_copy_prof (profiles_k_rec(ichannels_rec:ichannels_rec), profiles_ad(iprof:iprof))
      
      ENDDO

    ELSE
!
! PC-scores
!
      DO ipcscore = 1, npcscores
     
        emissivity_ad                = 0._jprb
        emissivity_out_ad            = 0._jprb

        CALL rttov_init_transmission (transmission_ad)
        CALL rttov_init_rad (radiancedata_ad)
      
        CALL rttov_init_pccomp (pccomp_ad)

        pccomp_ad%pcscores(ipcscore) = 1._jprb
      
        CALL rttov_init_prof (profiles_ad)
 
        CALL rttov_ad(                     &
              & errorstatus,               &
              & chanprof,                  &
              & opts,                      &
              & profiles,                  &
              & profiles_ad,               &
              & coefs,                     &
              & calcemis,                  &
              & emissivity,                &
              & emissivity_ad,             &
              & emissivity_out,            &
              & emissivity_out_ad,         &
              & transmission,              &
              & transmission_ad,           &
              & radiancedata,              &
              & radiancedata_ad,           &
              & pccomp       = pccomp,     &
              & pccomp_ad    = pccomp_ad,  &
              & channels_rec = channels_rec)
     
        IF (errorstatus /= 0) THEN
          err = errorstatus_fatal
          THROW(err.ne.0)
        ENDIF
     
        iprof = 1 + ((nprofiles * (ipcscore-1))/npcscores)
        CALL rttov_copy_prof (profiles_k_pc(ipcscore:ipcscore), profiles_ad(iprof:iprof))
     
      ENDDO

    ENDIF

  ENDIF

  IF (opts%addpc) THEN
    CALL rttov_alloc_pccomp (err, pccomp_ad, npcscores, 0_jpim, nchannels_rec = nchannels_rec)
    THROW(err.ne.0)
  ENDIF

  CALL rttov_alloc_transmission(err, transmission_ad, profiles(1)%nlevels-1_jpim, nchannels, 0_jpim)
  THROW(err.ne.0)

  CALL rttov_alloc_rad( &
        & err,                          &
        & nchannels,                    &
        & radiancedata_ad,              &
        & profiles(1)%nlevels - 1_jpim, &
        & 0_jpim)
  THROW(err.ne.0)

  CALL rttov_alloc_prof( &
        & err,                 &
        & nprofiles,           &
        & profiles_ad,         &
        & profiles(1)%nlevels, &
        & opts,                &
        & 0_jpim)
  THROW(err.ne.0)

CATCH

  errorstatus = err

END SUBROUTINE rttov_k_ad

