!
SUBROUTINE rttov_k_tl( &
            & errorstatus,      &
            & chanprof,         &
            & opts,             &
            & profiles,         &
            & profiles_k,       &
            & coefs,            &
            & calcemis,         &
            & emissivity,       &
            & emissivity_k,     &
            & emissivity_out,   &
            & transmission,     &
            & radiancedata,     &
            & pccomp,           &
            & profiles_k_pc,    &
            & profiles_k_rec,   &
            & channels_rec)
! Description:
!   Compute jacobian using tangent linear code
!
! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! Method:
!
! Current Code Owner: SAF NWP
!
! History:
! Version   Date     Comment
! -------   ----     -------
! 2010/03 Code cleaning Pascal Brunel, Philippe Marguinaud
!
!
! Code Description:
!   Language:           Fortran 90.
!   Software Standards: "European Standards for Writing and
!     Documenting Exchangeable Fortran 90 Code".
!INTF_OFF
#include "throw.h"
!INTF_ON

! Imported Type Definitions:
  USE rttov_types, ONLY :  &
       & rttov_coefs,       &
       & rttov_options,     &
       & profile_Type,      &
       & transmission_Type, &
       & rttov_chanprof,    &
       & radiance_Type,     & 
       & rttov_pccomp
  USE parkind1, ONLY : jpim, jprb, jplm
!INTF_OFF
  USE rttov_chain, ONLY :  &
       & chain,              &
       & pchain,             &
       & size_chain,         &
       & get_pointer_chain,  &
       & advance_chain
  USE rttov_test_k_mod, ONLY : &
    & make_chain_profile, &
    & free_chain_profile, &
    & assign_chain_profile
       
!INTF_ON
  IMPLICIT NONE
!subroutine arguments:
  TYPE(profile_Type  )   , INTENT(IN)   , TARGET :: profiles  (:)
  TYPE(rttov_chanprof)   , INTENT(IN)            :: chanprof  (:)
  TYPE(profile_Type  )   , INTENT(INOUT), TARGET :: profiles_k(size(chanprof))
! for calls from RTTOV_CLD_K and RTTOV_SCATT_K.
  TYPE(rttov_options )   , INTENT(IN)            :: opts
  TYPE(rttov_coefs   )   , INTENT(IN)            :: coefs
  LOGICAL(KIND=jplm)     , INTENT(IN)            :: calcemis        (size(chanprof))
  INTEGER(KIND=jpim)     , INTENT(OUT)           :: errorstatus
  REAL   (KIND=jprb)     , INTENT(INOUT)         :: emissivity      (size(chanprof))
  REAL   (KIND=jprb)     , INTENT(INOUT)         :: emissivity_k    (size(chanprof))
  REAL   (KIND=jprb)     , INTENT(OUT)           :: emissivity_out  (size(chanprof))
  TYPE(transmission_Type), INTENT(INOUT)         :: transmission                    ! in because of meme allocation
  TYPE(radiance_Type    ), INTENT(INOUT)         :: radiancedata                    ! in because of meme allocation
  TYPE(rttov_pccomp     ), OPTIONAL     , INTENT(INOUT)         :: pccomp
  TYPE(profile_Type     ), OPTIONAL     , INTENT(INOUT), TARGET :: profiles_k_pc(:)
  TYPE(profile_Type     ), OPTIONAL     , INTENT(INOUT), TARGET :: profiles_k_rec(:)
  INTEGER(KIND=jpim)     , OPTIONAL     , INTENT(IN)            :: channels_rec(:)
!INTF_END

#include "rttov_tl.interface"
#include "rttov_alloc_prof.interface"
#include "rttov_alloc_pccomp.interface"
#include "rttov_alloc_rad.interface"
#include "rttov_alloc_transmission.interface"
#include "rttov_init_rad.interface"
#include "rttov_init_pccomp.interface"

  TYPE(rttov_pccomp)      :: pccomp_tl
  TYPE(profile_Type), TARGET :: profiles_tl(size(profiles))
  REAL(KIND=jprb)         :: emissivity_tl    (size(chanprof))
  REAL(KIND=jprb)         :: emissivity_out_tl(size(chanprof))
  TYPE(transmission_Type) :: transmission_tl
  TYPE(radiance_Type    ) :: radiancedata_tl
  TYPE(chain),  POINTER :: chain_profiles_tl(:)
  TYPE(chain),  POINTER :: chain_profiles_k(:)
  TYPE(chain),  POINTER :: chain_profiles_k_pc(:)
  TYPE(chain),  POINTER :: chain_profiles_k_rec(:)
  TYPE(pchain), POINTER :: c_tl(:)
  TYPE(pchain), POINTER :: c_k(:)
  TYPE(pchain), POINTER :: c_k_pc(:)
  TYPE(pchain), POINTER :: c_k_in(:)
  INTEGER(KIND=jpim) :: err, i, nsize, ichan, iprof
  REAL(KIND=jprb), POINTER :: x
  INTEGER(KIND=jpim) :: nprofiles
  INTEGER(KIND=jpim) :: nchannels
  INTEGER(KIND=jpim) :: nchannels_rec
  INTEGER(KIND=jpim) :: npcscores

TRY

  nprofiles = size(profiles)
  nchannels = size(chanprof)


  CALL rttov_alloc_prof( &
        & err,                                                           &
        & nprofiles,                                                     &
        & profiles_tl,                                                   &
        & profiles(1)%nlevels,                                           &
        & opts,                                                          &
        & 1_jpim,                                                        &
        & coefs = coefs,                                                 &
        & init = .TRUE._jplm)
  THROW(err.ne.0)

  CALL rttov_alloc_rad( &
        & err,                          &
        & nchannels,                    &
        & radiancedata_tl,              &
        & profiles(1)%nlevels - 1_jpim, &
        & 1_jpim)
  THROW(err.ne.0)

  CALL rttov_alloc_transmission(err, transmission_tl, profiles(1)%nlevels-1_jpim, nchannels, 1_jpim)
  THROW(err.ne.0)

!
! Chain input profiles
!

  CALL make_chain_profile (err, chain_profiles_tl, c_tl, "PROFILES_TL", profiles_tl, .false._jplm)
  THROW(err.ne.0)

!
! Regular radiances jacobians
!
  CALL make_chain_profile (err, chain_profiles_k, c_k, "PROFILES_K", profiles_k, .true._jplm)
  THROW(err.ne.0)

!
  NULLIFY (chain_profiles_k_rec, c_k_in)
  nchannels_rec = -1_jpim

  NULLIFY (chain_profiles_k_pc, c_k_pc)
  npcscores = -1_jpim

  IF (opts%addpc) THEN

    npcscores = size(pccomp%pcscores)

    IF (opts%addradrec) THEN
!
! Reconstructed radiances jacobians
!
      nchannels_rec = size(pccomp%bt_pccomp)
      CALL make_chain_profile (err, chain_profiles_k_rec, c_k_in, "PROFILES_K_REC", profiles_k_rec, .true._jplm)
      THROW(err.ne.0)
    ELSE
!
! PC-scores jacobians
!
      CALL make_chain_profile (err, chain_profiles_k_pc, c_k_pc, "PROFILES_K_PC", profiles_k_pc, .true._jplm)
      THROW(err.ne.0)

    ENDIF

!
! Allocate a pccomp_tl
!
    CALL rttov_alloc_pccomp (err, pccomp_tl, npcscores, 1_jpim, nchannels_rec = nchannels_rec)
    THROW(err.ne.0)
  ENDIF

  CALL size_chain(nsize, chain_profiles_tl(1))! assume all profiles have the same size

  DO i = 1, nsize
    emissivity_tl         = 0._jprb
    emissivity_out_tl     = 0._jprb

    CALL rttov_init_rad (radiancedata_tl)
    IF (opts%addpc) THEN
      CALL rttov_init_pccomp (pccomp_tl)
    ENDIF

    DO iprof = 1, nprofiles
      CALL get_pointer_chain(c_tl(iprof)%p, x)
      x = 1._jprb
!      Print *, '-------', i
!      Call print_chain( 6, chain_profiles_tl(iprof) )
    ENDDO

    CALL rttov_tl( &
          & errorstatus,               &
          & chanprof,                  &
          & opts,                      &
          & profiles,                  &
          & profiles_tl,               &
          & coefs,                     &
          & calcemis,                  &
          & emissivity,                &
          & emissivity_tl,             &
          & emissivity_out,            &
          & emissivity_out_tl,         &
          & transmission,              &
          & transmission_tl,           &
          & radiancedata,              &
          & radiancedata_tl,           &
          & pccomp       = pccomp,     &
          & pccomp_tl    = pccomp_tl,  &
          & channels_rec = channels_rec)

    IF (errorstatus /= 0) THEN
      err = errorstatus_fatal
      THROW(err.ne.0)
    ENDIF

    DO iprof = 1, nprofiles
      CALL get_pointer_chain(c_tl(iprof)%p, x)
      x = 0._jprb
      CALL advance_chain(c_tl(iprof)%p)
    ENDDO

    IF (opts%switchrad) THEN
      IF (opts%addpc) THEN
        CALL assign_chain_profile (c_k, radiancedata_tl%total)
        IF (opts%addradrec) THEN
          CALL assign_chain_profile (c_k_in, pccomp_tl%bt_pccomp)
        ELSE
          CALL assign_chain_profile (c_k_pc, pccomp_tl%pcscores)
        ENDIF
      ELSE
        CALL assign_chain_profile (c_k, radiancedata_tl%bt)
      ENDIF
    ELSE
      CALL assign_chain_profile (c_k, radiancedata_tl%total)
      IF (opts%addpc) THEN
        IF (opts%addradrec) THEN
          CALL assign_chain_profile (c_k_in, pccomp_tl%clear_pccomp)
        ELSE
          CALL assign_chain_profile (c_k_pc, pccomp_tl%pcscores)
        ENDIF
      ENDIF
    ENDIF

   
  ENDDO

  CALL free_chain_profile (err, chain_profiles_tl, c_tl)
  THROW(err.ne.0)

  CALL free_chain_profile (err, chain_profiles_k, c_k)
  THROW(err.ne.0)

  IF (opts%addpc) THEN
    IF (opts%addradrec) THEN
      CALL free_chain_profile (err, chain_profiles_k_rec, c_k_in)
      THROW(err.ne.0)
    ELSE
      CALL free_chain_profile (err, chain_profiles_k_pc, c_k_pc)
      THROW(err.ne.0)
    ENDIF
  ENDIF

  
  DO ichan = 1, nchannels
    emissivity_tl        = 0._jprb
    emissivity_tl(ichan) = 1._jprb
    CALL rttov_tl( &
          & errorstatus,       &
          & chanprof,          &
          & opts,              &
          & profiles,          &
          & profiles_tl,       &
          & coefs,             &
          & calcemis,          &
          & emissivity,        &
          & emissivity_tl,     &
          & emissivity_out,    &
          & emissivity_out_tl, &
          & transmission,      &
          & transmission_tl,   &
          & radiancedata,      &
          & radiancedata_tl,   &
          & pccomp       = pccomp,     &
          & pccomp_tl    = pccomp_tl,  &
          & channels_rec = channels_rec)
    IF (errorstatus /= 0) THEN
      err = errorstatus_fatal
      THROW(err.ne.0)
    ENDIF

    IF (opts%switchrad) THEN
      emissivity_k(ichan) = radiancedata_tl%bt(ichan)
    ELSE
      emissivity_k(ichan) = radiancedata_tl%total(ichan)
    ENDIF
  ENDDO  

  IF (opts%addpc) THEN
    CALL rttov_alloc_pccomp (err, pccomp_tl, npcscores, 0_jpim, nchannels_rec = nchannels_rec)
    THROW(err.ne.0)
  ENDIF

  CALL rttov_alloc_transmission(err, transmission_tl, profiles(1)%nlevels-1_jpim, nchannels, 0_jpim)
  THROWM(err.ne.0,"Deallocation of transmission_tl failed")

  CALL rttov_alloc_rad( &
        & err,                          &
        & nchannels,                    &
        & radiancedata_tl,              &
        & profiles(1)%nlevels - 1_jpim, &
        & 0_jpim)
  THROW(err.ne.0)

  CALL rttov_alloc_prof( &
        & err,                 &
        & nprofiles,           &
        & profiles_tl,         &
        & profiles(1)%nlevels, &
        & opts,                &
        & 0_jpim)
  THROW(err.ne.0)


CATCH

  errorstatus = err

END SUBROUTINE rttov_k_tl

