Program test_weighting_fn_dev
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2010, EUMETSAT, All Rights Reserved.
  !
  !     *************************************************************
  !
  ! Test program showing the impact of interpolation on weighting functions
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date        Comment
  ! -------   ----        -------
  !  1.0    20/06/2011   Created (J Hocking)
  !
  ! Code Description:
  !   Language:          Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !

  Use rttov_const, Only :  &
       & errorstatus_success,&
       & errorstatus_fatal

  Use rttov_types, Only :   &
       & rttov_options,     &
       & rttov_coefs,       &
       & rttov_traj,        &
       & profile_Type,      &
       & transmission_Type, &
       & radiance_Type,     &
       & rttov_chanprof,    &
       & sskin_type,        &
       & s2m_type

  Use parkind1, Only : jpim, jprb, jplm

  !
  Implicit None
  !
#include "rttov_direct.interface"
#include "rttov_calc_weighting_fn.interface"
#include "rttov_read_coefs.interface"
#include "rttov_init_coefs.interface"
#include "rttov_dealloc_coefs.interface"
#include "rttov_alloc_traj.interface"
#include "rttov_alloc_rad.interface"
#include "rttov_alloc_transmission.interface"
#include "rttov_alloc_prof.interface"
#include "rttov_errorreport.interface"


  !--------------------------
  !
  Integer(Kind=jpim) :: file_id=20
  Integer(Kind=jpim) :: Err_Unit        ! Logical error unit (<0 for default)

  Integer(Kind=jpim) :: err
  
  Integer(Kind=jpim)              :: setup_errorstatus  ! setup return code
  Type(rttov_options)             :: opts               ! Options structure
  Type(rttov_coefs), Target       :: coefs              ! Coefficients structure ("Target" is requried)
  
  Integer(Kind=jpim)                :: rttov_errorstatus  ! rttov error return code
  Integer(Kind=jpim)                :: nchan, nprof, nchanprof
  Type(rttov_chanprof), Allocatable :: chanprof(:)
  Type(profile_Type), Allocatable   :: profiles(:)    
  Logical(Kind=jplm), Allocatable   :: calcemis(:)
  Real(Kind=jprb), Allocatable      :: emissivity_in (:)
  Real(Kind=jprb), Allocatable      :: emissivity_out (:)
  Type(transmission_Type)           :: transmission ! transmittances and layer optical depths
  Type(radiance_Type)               :: radiance
  Type(rttov_traj)                  :: traj
  
  Integer(Kind=jpim) :: alloc_status(20)
  Integer(Kind=jpim) :: errorstatus
  Character (len=80) :: errMessage
  Character (len=21)  :: NameOfRoutine = 'test_weighting_fn_dev'

  Integer(Kind=jpim) :: nlevels, ncoeflevels
  Integer(Kind=jpim) :: asw          ! allocate or deallocate switch
  
  Integer(Kind=jpim) :: j, ilev
  Integer            :: ios

  Real(Kind=jprb), Allocatable  :: weighting_fn(:)
  
  Type(sskin_type) :: k0
  Type(s2m_type)   :: s0
  
  Real(Kind=jprb) :: elevation, zenangle, azangle, sunzenangle, sunazangle, latitude
  
  Namelist / angles / zenangle, azangle, sunzenangle, sunazangle, latitude
  Namelist / s2m / s0
  Namelist / skin / k0
  Namelist / elev / elevation

  Character(len=128), Parameter :: datadir = 'tests.0/amsua/weighting_fn/in/'
  Character(len=128), Parameter :: profdir = trim(datadir)//'profiles/001/'
  
  Character(len=128), Parameter :: coeffile = '../rtcoef_rttov10/rttov7pred51L/rtcoef_noaa_15_amsua.dat'
  Integer(Kind=jpim), Parameter :: channel = 7_jpim   ! Channel for which to output weighting function
  
  !- End of header --------------------------------------------------------

  errorstatus     = 0_jpim
  alloc_status(:) = 0_jpim

  nlevels = 101_jpim
  nchan   = 15_jpim
  nprof   = 1_jpim
  
  ! Set flags
  opts % addrefrac = .TRUE.   ! include refraction in path calc
  opts % addinterp = .TRUE.   ! Allow interpolation of input profile 

  !Pack channels and emmissivity arrays
  nchanprof = nchan
  Allocate(chanprof(nchanprof))
  Allocate(calcemis(nchanprof))
  Allocate(emissivity_in(nchanprof))
  Allocate(emissivity_out(nchanprof))
  
  ! Build the list of profile indices
  Do  j = 1,nchan
    chanprof(j)%prof = 1_jpim
    chanprof(j)%chan = j
  End Do
  calcemis(:) = .TRUE.
    
  call rttov_read_coefs( errorstatus, coefs, opts, channels = chanprof(:)%chan, &
                         file_coef=coeffile)
  If( errorstatus /= errorstatus_success ) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "allocation error for coefs")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  Endif
  
  Call rttov_init_coefs  ( errorstatus, opts, coefs )
  If( errorstatus /= errorstatus_success ) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "initialisation error for coefs")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  Endif
                         
  ! Do allocation of input profile arrays with the number of levels.
  asw = 1_jpim ! allocate
  allocate( profiles(1),stat= alloc_status(1))
  profiles(1) % nlevels = nlevels
  call rttov_alloc_prof(         &
      & errorstatus,             &
      & nprof,                   &
      & profiles,                &
      & nlevels,                 &
      & opts,                    &
      & asw,                     &
      & coefs = coefs,           &
      & init = .true._jplm       )
  If( errorstatus /= errorstatus_success ) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error for profile arrays")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  Endif
  
  Open( file_id, file = Trim( profdir )//'atm/p.txt', form = 'formatted', status = 'old', iostat = ios )
  Read( file_id, *, iostat = ios ) profiles(1) % p(:)
  Close( file_id )

  Open( file_id, file = Trim( profdir )//'atm/t.txt', form = 'formatted', status = 'old', iostat = ios )
  Read( file_id, *, iostat = ios ) profiles(1) % t(:)
  Close( file_id )

  Open( file_id, file = Trim( profdir )//'atm/q.txt', form = 'formatted', status = 'old', iostat = ios )
  Read( file_id, *, iostat = ios ) profiles(1) % q(:)
  Close( file_id )
  
  Open( file_id, file = Trim( profdir )//'ground/skin.txt', form = 'formatted', status = 'old', iostat = ios )
  Read( file_id, nml = skin, iostat = ios )
  Close( file_id )
  profiles(1) % skin = k0
  
  Open( file_id, file = Trim( profdir )//'ground/s2m.txt', form = 'formatted', status = 'old', iostat = ios )
  Read( file_id, nml = s2m, iostat = ios )
  Close( file_id )
  profiles(1) % s2m = s0
  
  Open( file_id, file = Trim( profdir )//'ground/elevation.txt', form = 'formatted', status = 'old', iostat = ios )
  Read( file_id, nml = elev, iostat = ios )
  Close( file_id )
  profiles(1) % elevation = elevation

  Open( file_id, file = Trim( profdir )//'angles.txt', form = 'formatted', status = 'old', iostat = ios )
  Read( file_id, nml = angles, iostat = ios )
  Close( file_id )
  profiles(1) % zenangle = zenangle
  profiles(1) % azangle  = azangle
  profiles(1) % latitude = latitude
  

  ! allocate radiance results arrays with number of channels
  asw = 1_jpim ! allocate
  call rttov_alloc_rad( &
      & errorstatus,    &
      & nchanprof,      &
      & radiance,       &
      & nlevels-1_jpim,      &
      & asw)
  If( errorstatus /= errorstatus_success) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error for radiance arrays")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  Endif
  
  ! allocate transmittance structure
  call rttov_alloc_transmission( &
      & errorstatus,             &
      & transmission,            &
      & nlevels-1_jpim,          &
      & nchanprof,               &
      & asw,                     &
      & init = .true._jplm)  
  If( errorstatus /= errorstatus_success) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error for transmission arrays")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  Endif

  call rttov_alloc_traj( &
      & errorstatus, &
      & 1_jpim,      &
      & nchan,       &
      & opts,        &
      & nlevels,     &
      & coefs,       &
      & asw,         &
      & traj)
  If( errorstatus /= errorstatus_success) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error for traj structure")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  Endif

  ! Call RTTOV forward model
  call rttov_direct(         &
        & rttov_errorstatus, &! out   error flag
        & chanprof,          &! in    channel and profile index structure
        & opts,              &! in    options structure
        & profiles,          &! in    profile array
        & coefs,             &! in    coefficients strucutre
        & calcemis,          &! in    flag for intermal emissivity calc
        & emissivity_in,     &! in    input emissivities per channel
        & emissivity_out,    &! out   emissivities used by RTTOV per channel
        & transmission,      &! inout computed transmittances
        & radiance,          &! inout computed radiances
        & traj=traj)
   
  If ( rttov_errorstatus /= errorstatus_success ) Then
     Write ( 0, * ) 'rttov_direct error'
     Stop
  End If

  
  ! Write out pressures and weighting functions on coef levels for channel 7

  ncoeflevels = size(coefs % coef % ref_prfl_p(:))
  allocate(weighting_fn(ncoeflevels-1))
  
  call rttov_calc_weighting_fn(errorstatus,                                   &
                               coefs % coef % ref_prfl_p(:),                  &
                               traj % opdp_path_coef % atm_level(:,channel),  &
                               weighting_fn(:))
  If( errorstatus /= errorstatus_success) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "error in weighting function calculation")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  Endif

  Open(file_id,file='weighting_fn_coef_levels.dat',status='unknown',form='formatted',iostat=ios)
  If( ios /= 0 ) Then
     Write(*,*) 'error opening the output file ios= ',ios
     Stop
  Endif

  Write(file_id, '(i3)') ncoeflevels
  Do ilev = 2, ncoeflevels
    Write(file_id, '(2f10.4)') coefs % coef % ref_prfl_p(ilev), weighting_fn(ilev-1)
  End Do
  
  Close(file_id,iostat=ios)
  If( ios /= 0 ) Then
    Write(*,*) 'error closing the output file ios= ',ios
    Stop
  Endif

  deallocate(weighting_fn)
  

  ! Write out pressures and weighting functions on user levels
  allocate(weighting_fn(nlevels-1))

  call rttov_calc_weighting_fn(errorstatus,                             &
                               profiles(1) % p(:),                      &
                               traj % opdp_path % atm_level(:,channel), &
                               weighting_fn(:))
  If( errorstatus /= errorstatus_success) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "error in weighting function calculation")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  Endif
  
  Open(file_id,file='weighting_fn_user_levels.dat',status='unknown',form='formatted',iostat=ios)
  If( ios /= 0 ) Then
     Write(*,*) 'error opening the output file ios= ',ios
     Stop
  Endif

  Write(file_id, '(i3)') nlevels
  Do ilev = 2, nlevels
    Write(file_id, '(2f10.4)') profiles(1) % p(ilev), weighting_fn(ilev-1)
  End Do
  
  Close(file_id,iostat=ios)
  If( ios /= 0 ) Then
    Write(*,*) 'error closing the output file ios= ',ios
    Stop
  Endif

  deallocate(weighting_fn)
  
  
  ! Deallocate arrays
  deallocate( chanprof,       stat=alloc_status(1))
  deallocate( emissivity_in,  stat=alloc_status(3))
  deallocate( emissivity_out, stat=alloc_status(4))
  deallocate( calcemis,       stat=alloc_status(5))

  If( any(alloc_status /= 0) ) then
    errorstatus = errorstatus_fatal
    Write( errMessage, '( "mem deallocation error")' )
    Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
    Stop
  End If

  asw = 0_jpim ! deallocate structures
  
  call rttov_alloc_traj(errorstatus,1_jpim,nchan,opts,nlevels,coefs,asw,traj)
  If(errorstatus /= errorstatus_success) Then
    Write( errMessage, '( "trajectory deallocation error")' )
    Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
  Endif
      
  call rttov_alloc_rad (errorstatus,nchan,radiance,nlevels-1_jpim,asw)
  If(errorstatus /= errorstatus_success) Then
    Write( errMessage, '( "radiance deallocation error")' )
    Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
  Endif

  call rttov_alloc_transmission (errorstatus,transmission,nlevels-1_jpim,nchan,asw)
  If(errorstatus /= errorstatus_success) Then
    Write( errMessage, '( "radiance deallocation error")' )
    Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
  Endif
  
  call rttov_alloc_prof (errorstatus,nprof,profiles,nlevels,opts,asw)
  deallocate(profiles,stat=alloc_status(1))
  If(errorstatus /= errorstatus_success .or. alloc_status(1) /= 0) Then
    Write( errMessage, '( "profile deallocation error")' )
    Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
  Endif

  Call rttov_dealloc_coefs (errorstatus, coefs)
  If(errorstatus /= errorstatus_success) Then
    Write( errMessage, '( "coefs deallocation error")' )
    Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
  Endif

End Program test_weighting_fn_dev
